/**
  ******************************************************************************
  * @file    demo_audio.c
  * @author  MCD Application Team
  * @version V1.2.1
  * @date    19-May-2011
  * @brief   This file contains all the functions for the Voice demo.
  ******************************************************************************
  * @attention
  *
  */ 

// Includes ------------------------------------------------------------------
#include "menu.h"

// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------
// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------
extern      uint32_t RecordDataLength;        /* Voice recorder global data length */
extern __IO uint32_t SPIFlashWriteAddr;       /* SPI FLASH write address global address */
extern CONST uint8_t AlarmWave[];

#ifdef _COSMIC_
__IO uint8_t InfoTab[5] @0x300;
#elif defined _RAISONANCE_
at 0x300 __IO uint8_t InfoTab[5];
#elif defined _IAR_
__no_init uint8_t InfoTab[5] @0x300;
#endif /* _COSMIC_ */

uint32_t SectorOffset = 0;                    /* SPI FLASH voice recording sector offset for erasing */
uint32_t SelectedMaxRecordDataLength = 0;     /* Selected max SPI FLASH size dedicated */
__IO uint8_t  RecordStatus = DISABLE;         /* Voice recorder global Status */
__IO uint8_t  PlayStatus = DISABLE;           /* Voice Player global Status */
__IO uint8_t  AlarmPlayStatus = DISABLE;      /* Alarm Player global Status */
__IO uint8_t  VoiceRecStop = RESET;           /* Stop Voice Recorder  Flag*/
__IO uint8_t  VoicePlayStop = RESET;          /* Stop Voice Player Flag */
__IO uint8_t  AlarmPlayStop = RESET;          /* Stop Alarm Player Flag */
__IO uint32_t WaveDataLength;                 /* Wave player global data lenght*/
__IO uint32_t WaveCounter = 0; /* Wave player and voice recorder global index */
// for voice recording 
// Private function prototypes -----------------------------------------------
// Private functions ---------------------------------------------------------
  // * @addtogroup Demo_Functions

/**
  * @brief  Record voice.
  * @param  None
  * @retval None
  */
void VOICERECORDER_Exec(void)
{
  JOYState_TypeDef Key;
  
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_Print("Voice REC Start");
  
  // Infinite loop
  while (1)
  {
    // Wait Joystick pressed 
    Key = ReadJoystick();
    if (Key == JOY_UP)
    {
      MenuUpdate(Key);
      return;
    }

    // Display "Press SEL button if you want to stop recording" message 
    if (Key == JOY_SEL)
    {
      // Disable RTC Alarm interrupt
      RTC_ITConfig(RTC_IT_ALRA, DISABLE);

      LCD_SetCursorPos(LCD_LINE2, 0);
      LCD_Print("SPI Flash Erase");
      
      // Init the SPI Flash 
      sFLASH_Init();

      // Erase SPI Flash sectors dedicated for Voice recording
      VoiceRec_SPIFlashErase();

      // Init Voice recorder 
      VoiceRec_Init();
      STM8_EVAL_LCD_Init();

      LCD_SetCursorPos(LCD_LINE1, 0);
      LCD_Print("Voice Recorder");

      LCD_SetCursorPos(LCD_LINE2, 0);
      LCD_Print("Record:PressSEL");
      
      // Enable RTC Alarm interrupt
      RTC_ITConfig(RTC_IT_ALRA, ENABLE);
      
      // Infinite loop
      while (1)
      {
        // Wait Joystick pressed 
        Key = ReadJoystick();
        if (Key == JOY_UP)
        {
          MenuUpdate(Key);
          return;
        }

        if (Key == JOY_SEL)
        {
          // Disable RTC Alarm interrupt
          RTC_ITConfig(RTC_IT_ALRA, DISABLE);
  
          LCD_SetCursorPos(LCD_LINE2, 0);
          LCD_Print("Stop:In 30s/UP ");
          
          // Init the SPI Flash 
          sFLASH_Init();

          // Select button to stop Playing 
          STM_EVAL_PBInit(BUTTON_UP, BUTTON_MODE_EXTI);

          // Start Voice recorder 
          VoiceRec_Start();

          // Record Voice until 30s/KEY
          VoiceRec_Recording();

          // stop Voice player 
          VoiceRec_Stop();
          // Select button to stop Playing 
          STM_EVAL_PBInit(BUTTON_UP, BUTTON_MODE_GPIO);

          STM8_EVAL_LCD_Init();
          
          MenuUpdate(JOY_UP);
          
          // Enable RTC Alarm interrupt
          RTC_ITConfig(RTC_IT_ALRA, ENABLE);
          
          return;
        }
      }
    }
  }
}
/**
  * @brief  Play the recorded voice.
  * @param  None
  * @retval None
  */
void VOICEPLAYER_Exec(void)
{
  JOYState_TypeDef Key;
  
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_Print("Recorded Voice ");
  
  // Infinite loop
  while (1)
  {
    // Wait Joystick pressed 
    Key = ReadJoystick();
    if (Key == JOY_UP)
    {
      MenuUpdate(Key);
      return;
    }

    // Display "Press SEL button if you want to stop recording" message 
    if (Key == JOY_SEL)
    {
      LCD_SetCursorPos(LCD_LINE1, 0);
      LCD_Print("Recorded Voice ");
      
      LCD_SetCursorPos(LCD_LINE1, 0);
      LCD_Print("      Play     ");

      // Enable RTC Alarm interrupt
      RTC_ITConfig(RTC_IT_ALRA, DISABLE);
      
      // Init the SPI Flash 
      sFLASH_Init();

      // Select button to stop Playing 
      STM_EVAL_PBInit(BUTTON_UP, BUTTON_MODE_EXTI);

      // Init voice player 
      VoicePlay_Init();

      // Start voice player
      VoicePlay_Start();

      // Play Voice until 30s/KEY
      VoicePlay_Playing();

      // stop Voice player 
      VoicePlay_Stop();

      // Select button to stop Playing 
      STM_EVAL_PBInit(BUTTON_UP, BUTTON_MODE_GPIO);

      STM8_EVAL_LCD_Init();
      
      MenuUpdate(JOY_UP);
      
      // Enable RTC Alarm interrupt
      RTC_ITConfig(RTC_IT_ALRA, ENABLE);
      return;
    }
  }
}
/**
  * @brief  Play the Alarm wave.
  * @param  None
  * @retval None
  */
void ALARMPLAYER_Exec(void)
{
  // Init voice player 
  VoiceAlarm_Init();

  // Start voice player
  VoiceAlarm_Start();

  VoiceAlarm_Playing();

  // stop Voice player 
  VoiceAlarm_Stop();
}
/**
  * @brief  Erase the SPI FLASH sectors reserved for voice recording
  * @param  None
  * @retval None
  */
void VoiceRec_SPIFlashErase(void)
{
  uint8_t sect = 0;

  // Initialize the SPI FLASH sector offset for erasing: voice recording 
  SectorOffset = RECORD_INFO_ADDR;

  // SPI FLASH Configuration 
  sFLASH_Init();

  // Set SPIFlashWriteAddr to the Recorded wave start read write address 
  SPIFlashWriteAddr = RECORD_READ_WRITED_ADDR;

  // Erase Flash sectors to store voice recording data 
  for (sect = SECTORS_NUMBER; sect != 0; sect--)
  {
    // Erase Flash sector 
    sFLASH_EraseSector(SectorOffset);
    // Select the next Flash sector address to erase 
    SectorOffset += SECTOR_SIZE;
  }
  RecordDataLength = 0;
}

/**
  * @brief  Initialize voice recording
  * @param  None
  * @retval None
  */
void VoiceRec_Init(void)
{
  // Reset WaveCounter value 
  WaveCounter = 0;

  // maximum data is for 30s recording
  SelectedMaxRecordDataLength = MAX_RECORD_DATA_LENGTH;

  // Voice recorder stopped flag is reset
  VoiceRecStop = RESET;

   ADC_DeInit(ADC1);
  // Initialise and configure ADC1 
  ADC_Init(ADC1, ADC_ConversionMode_Continuous, ADC_Resolution_8Bit, ADC_Prescaler_1);
  ADC_SamplingTimeConfig(ADC1, ADC_Group_SlowChannels, ADC_SamplingTime_384Cycles);

  // TIM1 is the trigger of ADC conversion
  ADC_ExternalTrigConfig(ADC1, ADC_ExtEventSelection_Trigger2, ADC_ExtTRGSensitivity_Rising);

  // Set TIM1 period to 22050 Hz 
  TIM1_SetAutoreload(0x2D6);

  // Master Mode selection: Update event 
  TIM1_SelectOutputTrigger(TIM1_TRGOSource_Update);

  // Enable TIM1 interrupts 
  TIM1_ITConfig(TIM1_IT_Update, ENABLE);
  TIM1_ITConfig(TIM1_IT_CC1, ENABLE);
}
/**
  * @brief  Start voice recording
  * @param  None
  * @retval None
  */
void VoiceRec_Start(void)
{
  // Enable ADC1 
  ADC_Cmd(ADC1, ENABLE);

  // Enable ADC1 Channel 15 
  ADC_ChannelCmd(ADC1, ADC_Channel_15, ENABLE);

  // Start TIM1 
  TIM1_Cmd(ENABLE);

  // Enable recording
  RecordStatus = ENABLE;
}

/**
  * @brief  Stop voice recording
  * @param  None
  * @retval None
  */

void VoiceRec_Stop(void)
{
  // Disable Recording
  RecordStatus = DISABLE;

  // Save data length
  RecordDataLength = WaveCounter;

  // Store Recorded data lenght in the SPI Flash 
  InfoTab[0] = RECORD_INFO_CODE_OK;
  InfoTab[4] = (uint8_t)RecordDataLength ;
  InfoTab[3] = (uint8_t)(RecordDataLength >> 8);
  InfoTab[2] = (uint8_t)(RecordDataLength >> 16);
  InfoTab[1] = (uint8_t)(RecordDataLength >> 24);

  // Load SPI Flash with the actual InfoTab 
  sFLASH_WriteBuffer(InfoTab, RECORD_INFO_ADDR, 5);

  // Disable TIM4 update interrupt 
  TIM1_ITConfig(TIM1_IT_Update, DISABLE);

  // Disable TIM1 
  TIM1_Cmd(DISABLE);

  // Stop conversion 
  ADC_Cmd(ADC1, DISABLE);
  
  // Unlock Data Memory 
  FLASH_Unlock(FLASH_MemType_Data);
  
  // Wait until Data EEPROM Area Unlocking Flag is set 
  while(FLASH_GetFlagStatus(FLASH_FLAG_DUL)== RESET)
  {
  }
  
  // Write Wave counter at @0x1000
  *(uint32_t*)0x1000 = WaveCounter;
}

/**
  * @brief  wait until Voice recording time finish or  a stop record action occurs
  * @param  None
  * @retval None
  */
void VoiceRec_Recording(void)
{
  // wait until 30s or Button Key pressed 
  while ((WaveCounter < SelectedMaxRecordDataLength) && (VoiceRecStop == RESET));

  // Reset the VoiceRecStop flag 
  VoiceRecStop = RESET;
}
/**
  * @brief  Initialize Voice Player
  * @param  None
  * @retval None
  */
void VoicePlay_Init(void)
{
  // Initialise the DAC 
  DAC_Init(DAC_Channel_1, DAC_Trigger_T4_TRGO, DAC_OutputBuffer_Enable);

  // Set TIM4 period to 22050 Hz 
  TIM4_TimeBaseInit(TIM4_Prescaler_4, 0xB5);

  // Enable TIM4 update interrupt 
  TIM4_ITConfig(TIM4_IT_Update, ENABLE);

  // Master Mode selection: Update event 
  TIM4_SelectOutputTrigger(TIM4_TRGOSource_Update);
}

/**
  * @brief  Start Voice Playing
  * @param  None
  * @retval None
  */
void VoicePlay_Start(void)
{
  // Read Wave Counter at @ 0x1000
  WaveCounter = *(uint32_t*)0x1000;
  
  // Enable TIM4 
  TIM4_Cmd(ENABLE);

  // Enable ADC1 
  DAC_Cmd(DAC_Channel_1, ENABLE);

  if (RecordDataLength == 0)
  {
    sFLASH_Init();
    sFLASH_ReadBuffer(InfoTab, RECORD_INFO_ADDR, 5);

    if (InfoTab[0] == RECORD_INFO_CODE_OK)
    {
        RecordDataLength = (uint32_t) InfoTab[4];
        RecordDataLength |= (uint32_t) InfoTab[3]<< 8;
        RecordDataLength |=  (uint32_t) InfoTab[2]<< 16;
        RecordDataLength |=  (uint32_t) InfoTab[1]<< 24;
    }
    else
    {
      VoicePlay_Stop();
      return;
    }
  }

  // Start fast reading process from SPI Flash
  sFLASH_StartReadSequence(RECORD_READ_WRITED_ADDR);

  // Set PlayStatus to Play the recorded voice 
  PlayStatus = ENABLE;

  // Reset WaveCounter value to play from the begining
  WaveCounter = 0;
}


/**
  * @brief  Stop Voice Playing
  * @param  None
  * @retval None
  */
void VoicePlay_Stop(void)
{
  // Disable Playing 
  PlayStatus = DISABLE;

  // Disable TIM4 update interrupt 
  TIM4_ITConfig(TIM4_IT_Update, DISABLE);

  // Disable TIM4 
  TIM4_Cmd(DISABLE);

  // Disable DAC 
  DAC_Cmd(DAC_Channel_1, DISABLE);

  // DeSelect the FLASH: Chip Select High 
  sFLASH_CS_HIGH();
}

/**
  * @brief  wait until Voice Playing time finish or a stop Playing action occurs
  * @param  None
  * @retval None
  */
void VoicePlay_Playing(void)
{
  // wait until 30s or Select key pressed 
  while ((WaveCounter < RecordDataLength) && (VoicePlayStop == RESET));

  // Reset the VoicePlayStop flag 
  VoicePlayStop = RESET;
}

/**
  * @brief  Start Alarm wave Playing
  * @param  None
  * @retval None
  */
void VoiceAlarm_Start(void)
{

  // Enable TIM4 
  TIM4_Cmd(ENABLE);

  // Enable ADC1 
  DAC_Cmd(DAC_Channel_1, ENABLE);

  // Set PlayStatus to Play the recorded voice 
  AlarmPlayStatus = ENABLE;

  // Reset WaveCounter value to play from the begining
  WaveCounter = 0;
}

/**
  * @brief  Initialize Voice Alarm  Player
  * @param  None
  * @retval None
  */
void VoiceAlarm_Init(void)
{
  VoicePlay_Init();
  // Set TIM4 period to 8000 Hz 
  TIM4_TimeBaseInit(TIM4_Prescaler_8, 0xFA);
}

/**
  * @brief  wait until Alarm stop action occurs
  * @param  None
  * @retval None
  */
void VoiceAlarm_Playing(void)
{
  uint8_t index = 0;

  for (index = 6; (index != 0) && (AlarmPlayStop == RESET); index--)
  {
    while ((WaveCounter != ALARM_BYTE_LENGTH));
    WaveCounter = 0;
  }

  AlarmPlayStop = RESET;
}
/**
  * @brief  Stop Voice Alarm  Playing
  * @param  None
  * @retval None
  */
void VoiceAlarm_Stop(void)
{
  // Disable Playing 
  AlarmPlayStatus = DISABLE;

  // Disable TIM4 update interrupt 
  TIM4_ITConfig(TIM4_IT_Update, DISABLE);

  // Disable TIM4 
  TIM4_Cmd(DISABLE);

  // Disable DAC 
  DAC_Cmd(DAC_Channel_1, DISABLE);
}



