/**
  ******************************************************************************
  * @file    demo_calendar.c
  * @author  MCD Application Team
  * @version V1.2.1
  * @date    19-May-2011
  * @brief   This file contains all the functions for the demo calendar.
  ******************************************************************************
  * @attention
  *
  */ 

// Includes ------------------------------------------------------------------
#include "menu.h"

// Private typedef -----------------------------------------------------------
typedef uint8_t* string;
// Private define ------------------------------------------------------------
#define ASCII_NUM_0         ((uint8_t)48)
#define ASCII_NUM_9         ((uint8_t)48+9)
// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------
// Peripheral InitStructure define 
RTC_InitTypeDef   RTC_InitStr;
RTC_TimeTypeDef   RTC_TimeStr;
RTC_DateTypeDef   RTC_DateStr;
RTC_AlarmTypeDef  RTC_AlarmStr;

#ifdef _COSMIC_
__IO uint8_t TempVar[16] @ 0x500;
#elif defined _RAISONANCE_
at 0x500 __IO uint8_t TempVar[16];
#elif defined _IAR_
__no_init __IO uint8_t TempVar[16] @0x500;
#endif /* _COSMIC_ */

__IO uint8_t LCDStringTime[16]       = "    00:00:00    ";
__IO uint8_t LCDStringDate[16]       = "   02/12/2010   ";

__IO uint8_t LCDStringAlarm[16]      = "    00:00:00    ";

__IO bool AlarmOccurs = FALSE;


// Days names definition 
CONST string WeekDayNames[7] = {"SUN", "MON", "TUE", "WED", "THU", "FRI", "SAT"};

// Months names definition 
CONST   uint8_t MonthNames[] = {'J', 'A', 'N', 'F', 'E', 'B', 'M', 'A', 'R', 'A', 'P', 'R',
                                  'M', 'A', 'Y', 'J', 'U', 'N', 'J', 'U', 'L', 'A', 'U', 'G',
                                  'S', 'E', 'P', 'O', 'C', 'T', 'N', 'O', 'V', 'D', 'E', 'C'};

// Private function prototypes -----------------------------------------------
// Private functions ---------------------------------------------------------
  // * @addtogroup Demo_Functions

/**
  * @brief  Reads digit entered by user, using menu navigation keys.
  * @param  None
  * @retval Digit value
  */
uint8_t ReadDigit(uint8_t* ptr, uint8_t ColBegin, uint8_t ValueMax, uint8_t ValueMin)
{
  __IO uint8_t tmpvalue, i = 0;
  JOYState_TypeDef Key;

  // Initialize tmpvalue 
  tmpvalue = ptr[ColBegin];
  
  // Display  
  tmpvalue -= 0x30;
  SetCursorPosition(ptr, LCD_LINE2, ColBegin);
  sprintf((char*)TempVar,"%c", ptr[ColBegin]);
  LCD_Print((uint8_t*)TempVar);
  
  // Endless loop 
  while (1)
  {
    // Check which key is pressed 
    Key = ReadJoystick();

    // If "UP" pushbutton is pressed 
    if (Key == JOY_UP)
    {
      // Increase the value of the digit 
      if (tmpvalue >= ValueMax)
      {
        tmpvalue = ValueMin;
      }
      else
      {
        tmpvalue++;
      }

      SetCursorPosition(ptr, LCD_LINE2, ColBegin);
      ptr[ColBegin] = (uint8_t)(tmpvalue + 0x30);
      sprintf((char*)TempVar,"%c", ptr[ColBegin]);
      LCD_Print((uint8_t*)TempVar);
    }

    // If "DOWN" pushbutton is pressed 
    if (Key == JOY_DOWN)
    {
      // Decrease the value of the digit 
      if ((tmpvalue <= ValueMin) || (tmpvalue > ValueMax))
      {
        tmpvalue = ValueMax;
      }
      else
      {
        tmpvalue--;
      }

      SetCursorPosition(ptr, LCD_LINE2, ColBegin);
      ptr[ColBegin] = (uint8_t)(tmpvalue + 0x30);
      sprintf((char*)TempVar,"%c", ptr[ColBegin]);
      LCD_Print((uint8_t*)TempVar);
    }

    // If "SEL" pushbutton is pressed 
    if (Key == JOY_SEL)
    {
      // Display new value 
      SetCursorPosition(ptr, LCD_LINE2, ColBegin);
  
      ptr[ColBegin] = (uint8_t)(tmpvalue + 0x30);
      sprintf((char*)TempVar,"%c", ptr[ColBegin]);
      LCD_Print((uint8_t*)TempVar);
      
      // Return the digit value and exit 
      return tmpvalue;
    }
    if (i < 5)
    {
      ToggleCursorPosition(ptr, LCD_LINE2, ColBegin);
      i++;
    }
  }
}
/**
  * @brief  Initializes calendar application.
  * @param  None
  * @retval None
  */
void Calendar_Init(void)
{
  RTC_InitStr.RTC_HourFormat = RTC_HourFormat_24;
  RTC_InitStr.RTC_AsynchPrediv = 0x7F;
  RTC_InitStr.RTC_SynchPrediv = 0x00FF;
  RTC_Init(&RTC_InitStr);

  RTC_AlarmStr.RTC_AlarmTime.RTC_H12     = RTC_H12_AM;
  RTC_AlarmStr.RTC_AlarmTime.RTC_Hours   = 00;
  RTC_AlarmStr.RTC_AlarmTime.RTC_Minutes = 00;
  RTC_AlarmStr.RTC_AlarmTime.RTC_Seconds = 00;
  RTC_AlarmStr.RTC_AlarmMask = RTC_AlarmMask_DateWeekDay;

  RTC_SetAlarm(RTC_Format_BIN, &RTC_AlarmStr);
  RTC_ITConfig(RTC_IT_ALRA, ENABLE);
  RTC_AlarmCmd(ENABLE);

  RTC_TimeStr.RTC_H12     = RTC_H12_AM;
  RTC_TimeStr.RTC_Hours   = 00;
  RTC_TimeStr.RTC_Minutes = 00;
  RTC_TimeStr.RTC_Seconds = 00;
  RTC_SetTime(RTC_Format_BIN, &RTC_TimeStr);

  RTC_DateStr.RTC_WeekDay = RTC_Weekday_Monday;
  RTC_DateStr.RTC_Date = 14;
  RTC_DateStr.RTC_Month = RTC_Month_February;
  RTC_DateStr.RTC_Year = 9;
  RTC_SetDate(RTC_Format_BIN, &RTC_DateStr);
}
/**
  * @brief  Update the time entered by user, using demoboard keys.
  * @param  None
  * @retval None
  */
void Time_Regulate(void)
{
  uint8_t tmphh = 0, tmpmm = 0, tmpss = 0;

  // Read time hours 
  tmphh = ReadDigit((uint8_t*)LCDStringTime, 4, 0x2, 0x0);
  if (tmphh == 2)
  {
    if (LCDStringTime[5] > 0x03 + 0x30)
    {
      LCDStringTime[5] = 0 + 0x30;
    }
    tmphh = (uint8_t)(tmphh * 10 + ReadDigit((uint8_t*)LCDStringTime, 5,  0x3, 0x0));
  }
  else
  {
    tmphh = (uint8_t)(tmphh * 10 + ReadDigit((uint8_t*)LCDStringTime, 5, 0x9, 0x0));
  }
  // Read time  minutes 
  tmpmm = ReadDigit((uint8_t*)LCDStringTime, 7, 5, 0x0);
  tmpmm = (uint8_t)(tmpmm * 10 + ReadDigit((uint8_t*)LCDStringTime, 8, 0x9, 0x0));
  
  // Read time seconds 
  tmpss = ReadDigit((uint8_t*)LCDStringTime, 10, 5, 0x0);
  tmpss = (uint8_t)(tmpss * 10 + ReadDigit((uint8_t*)LCDStringTime, 11, 0x9, 0x0));

  // Set the Time 
  RTC_TimeStr.RTC_Hours   = tmphh;
  RTC_TimeStr.RTC_Minutes = tmpmm;
  RTC_TimeStr.RTC_Seconds = tmpss;
  RTC_SetTime(RTC_Format_BIN, &RTC_TimeStr);
}
/**
  * @brief  Update and display the time entered by user, using demoboard keys.
  * @param  None
  * @retval None
  */
void Time_Adjust(void)
{
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_Print("  Time Adjust  ");
  
  RTC_SetTime(RTC_Format_BIN, &RTC_TimeStr);
  
  // Display the current time 
  Time_Display();
  
  // Change the current time 
  Time_Regulate();

  // Execute MenuUpdate function to display upper menu 
  MenuUpdate(JOY_UP);

}
/**
  * @brief  Displays the current time.
  * @param  None
  * @retval None
  */
void Time_Display(void)
{  
  // Get the current Time
  RTC_GetTime(RTC_Format_BCD, &RTC_TimeStr);

  // Fill the LCDString fields with the current Time
  LCDStringTime[4] = (uint8_t)(((uint8_t)(RTC_TimeStr.RTC_Hours & 0xF0) >> 4) + ASCII_NUM_0);
  LCDStringTime[5] = (uint8_t)(((uint8_t)(RTC_TimeStr.RTC_Hours & 0x0F)) + ASCII_NUM_0);

  LCDStringTime[7] = (uint8_t)(((uint8_t)(RTC_TimeStr.RTC_Minutes & 0xF0) >> 4) + ASCII_NUM_0);
  LCDStringTime[8] = (uint8_t)(((uint8_t)(RTC_TimeStr.RTC_Minutes & 0x0F)) + (uint8_t)ASCII_NUM_0);

  LCDStringTime[10] = (uint8_t)(((uint8_t)(RTC_TimeStr.RTC_Seconds & 0xF0) >> 4) + ASCII_NUM_0);
  LCDStringTime[11] = (uint8_t)(((uint8_t)(RTC_TimeStr.RTC_Seconds & 0x0F)) + ASCII_NUM_0);

  // Print the Time Calendar on the LCD
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_Print((uint8_t*)LCDStringTime);
}
/**
  * @brief  Shows the current time (HH/MM/SS) on LCD.
  * @param  None
  * @retval None
  */
void Time_Show(void)
{
  JOYState_TypeDef Key;
  
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_Print("   Time Show   ");

  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_Print("               ");
  
  // Wait a press on "UP" pushbuttons 
  while (1)
  {
    // Wait Joystick pressed 
    Key = ReadJoystick();
    if (Key == JOY_UP)
    {
      MenuUpdate(Key);
      return;
    }
    // Display current time 
    Time_Display();
  }
}
/**
  * @brief  Sets the date entered by user, using menu navigation keys.
  * @param  None
  * @retval None
  */
void Date_Regulate(void)
{
  uint8_t tmpmonth = 0, tmpday = 0, tmp = 0;
  uint16_t tmpyear = 0;

  // Read date months 
  tmpmonth = ReadDigit((uint8_t*)LCDStringDate, 3, 1, 0x0);
  tmp = (uint8_t)(RTC_DateStr.RTC_Month % 10);
  if (tmpmonth == 1)
  {
    if (tmp > 2)
    {
      LCDStringDate[4] = 0 + 0x30;
    }
    tmpmonth = (uint8_t)(tmpmonth * 10 + ReadDigit((uint8_t*)LCDStringDate, 4, 2, 0x0));
  }
  else
  {
    tmpmonth = (uint8_t)(tmpmonth * 10 + ReadDigit((uint8_t*)LCDStringDate, 4, 9, 0x1));
  }

  // Read date days 
  if (tmpmonth == 2)
  {
    tmpday = ReadDigit((uint8_t*)LCDStringDate, 6, 2, 0x0);
  }
  else
  {
    tmpday = ReadDigit((uint8_t*)LCDStringDate, 6, 3, 0x0);
  }

  if (tmpday == 3)
  {
    tmp = (uint8_t)(RTC_DateStr.RTC_Date % 10);

    if (tmp >= 1)
    {
      LCDStringDate[7] = 0 + 0x30;
    }

    if (tmpmonth == 1 || tmpmonth == 3 || tmpmonth == 5 || tmpmonth == 7 ||
        tmpmonth == 8 || tmpmonth == 10 || tmpmonth == 12)
    {

      tmpday = (uint8_t)(tmpday * 10 + ReadDigit((uint8_t*)LCDStringDate, 7, 1, 0x0));
    }
    else
    {
      tmpday = (uint8_t)(tmpday * 10 + ReadDigit((uint8_t*)LCDStringDate, 7, 0, 0x0));
    }
  }
  else
  {
    if (tmpday == 0)
    {
      tmpday = (uint8_t)(tmpday * 10 + ReadDigit((uint8_t*)LCDStringDate, 7, 9, 0x1));
    }
    else
    {
      tmpday = (uint8_t)(tmpday * 10 + ReadDigit((uint8_t*)LCDStringDate, 7, 9, 0x0));
    }
  }

  // Read date years 
  tmpyear = ReadDigit((uint8_t*)LCDStringDate, 11, 9, 0x0);
  tmpyear = tmpyear * 10 + ReadDigit((uint8_t*)LCDStringDate, 12, 9, 0x0);

  if (tmpmonth == 2 && tmpday == 29 && (tmpyear % 4) != 0)
  {
    tmpday = 28;
  }

  // Return the date value to store in date structure 
  RTC_DateStr.RTC_Year = (uint8_t)tmpyear;
  RTC_DateStr.RTC_Month = (RTC_Month_TypeDef)tmpmonth;
  RTC_DateStr.RTC_Date = tmpday;
  RTC_SetDate(RTC_Format_BIN, &RTC_DateStr);
}

/**
  * @brief  Adjusts the current date (MM/DD/YYYY).
  * @param  None
  * @retval None
  */
void Date_Adjust(void)
{
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_Print("  Date Adjust  ");

  // Display the current Date 
  Date_Display();

  // Change the current Date 
  Date_Regulate();

  // Execute MenuUpdate function to display upper menu 
  MenuUpdate(JOY_UP);
}
/**
  * @brief  Displays the current date.
  * @param  - Type: specifies the date format. If Type = 1 then show
 *           (MM/DD/YYYY) else (MM/DD/YY).
  * @retval None
  */
void Date_Display(void)
{
  // Get the current Date 
  RTC_GetDate(RTC_Format_BCD, &RTC_DateStr);

  LCDStringDate[3] = (uint8_t) (((uint8_t)(RTC_DateStr.RTC_Month & 0xF0) >> 4) + ASCII_NUM_0);
  LCDStringDate[4] = (uint8_t) (((uint8_t)(RTC_DateStr.RTC_Month & 0x0F)) + ASCII_NUM_0);

  LCDStringDate[6] = (uint8_t) (((uint8_t)(RTC_DateStr.RTC_Date & 0xF0) >> 4) + ASCII_NUM_0);
  LCDStringDate[7] = (uint8_t) (((uint8_t)(RTC_DateStr.RTC_Date & 0x0F)) + ASCII_NUM_0);

  LCDStringDate[11] = (uint8_t) (((uint8_t)(RTC_DateStr.RTC_Year & 0xF0) >> 4) + ASCII_NUM_0);
  LCDStringDate[12] = (uint8_t) (((uint8_t)(RTC_DateStr.RTC_Year & 0x0F)) + ASCII_NUM_0);

  // Print the Date Calendar on the LCD

  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_Print((uint8_t*)LCDStringDate);
}
/**
  * @brief  Shows date (MM/DD/YYYY) on LCD.
  * @param  None
  * @retval None
  */
void Date_Show (void)
{
  JOYState_TypeDef Key;
  uint8_t tmpdate, tmpyear = 0;
  
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_Print("   Date Show   ");
  
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_Print("               ");

  // Wait a press on "JOY_UP" pushbuttons 
  while (1)
  {
    // Wait Joystick pressed 
    Key = ReadJoystick();
    if (Key == JOY_UP)
    {
      MenuUpdate(Key);
      return;
    }
    
    // Get the current Date 
    RTC_GetDate(RTC_Format_BIN, &RTC_DateStr);

    LCD_SetCursorPos(LCD_LINE2, 0);
    LCD_Print(WeekDayNames[DayOfWeek((RTC_DateStr.RTC_Year +2000), RTC_DateStr.RTC_Month,  RTC_DateStr.RTC_Date)]);
    
    LCD_Print(" ");

    sprintf((char*)TempVar,"%c",MonthNames[(((RTC_DateStr.RTC_Month) -1) * 3)]);
    LCD_Print((uint8_t*)TempVar);
    
    sprintf((char*)TempVar,"%c",MonthNames[((((RTC_DateStr.RTC_Month) -1) * 3) + 1)]);
    LCD_Print((uint8_t*)TempVar);
    
    sprintf((char*)TempVar,"%c",MonthNames[((((RTC_DateStr.RTC_Month) -1) * 3) + 2)]);
    LCD_Print((uint8_t*)TempVar);
    
    LCD_Print(" ");
    
    tmpdate = (uint8_t)((RTC_DateStr.RTC_Date / 10) + 0x30);
    sprintf((char*)TempVar,"%c",tmpdate);
    LCD_Print((uint8_t*)TempVar);
    
    tmpdate = (uint8_t)((RTC_DateStr.RTC_Date % 10) + 0x30);
    sprintf((char*)TempVar,"%c",tmpdate);
    LCD_Print((uint8_t*)TempVar);
    
    LCD_Print(" 20");
    
    tmpyear = (uint8_t)(((RTC_DateStr.RTC_Year % 1000) % 100) / 10 + 0x30);
    sprintf((char*)TempVar,"%c", tmpyear);
    LCD_Print((uint8_t*)TempVar);
    
    tmpyear = (uint8_t)(((RTC_DateStr.RTC_Year % 1000) % 10) % 10 + 0x30);
    sprintf((char*)TempVar,"%c", tmpyear);
    LCD_Print((uint8_t*)TempVar);
  }
}
/**
  * @brief  Sets the alarm entered by user, using menu navigation keys.
  * @param  None
  * @retval None
  */
void Alarm_Regulate(void)
{
  uint8_t alarmhh = 0, alarmmm = 0, alarmss = 0;

  // Read alarm hours 
  alarmhh = ReadDigit((uint8_t*)LCDStringAlarm, 4, 0x2, 0x0);

  if (alarmhh == 0x2)
  {
    if (LCDStringAlarm[5] > 0x03 + 0x30)
    {
      LCDStringAlarm[5] = 0 + 0x30;
    }
    alarmhh = (uint8_t)(alarmhh * 10 + ReadDigit((uint8_t*)LCDStringAlarm, 5, 0x3, 0x0));
  }
  else
  {
    alarmhh = (uint8_t)(alarmhh * 10 + ReadDigit((uint8_t*)LCDStringAlarm, 5, 0x9, 0x0));
  }

  // Read alarm minutes 
  alarmmm = ReadDigit((uint8_t*)LCDStringAlarm, 7, 0x5, 0x0);
  alarmmm  = (uint8_t)(alarmmm * 10 + ReadDigit((uint8_t*)LCDStringAlarm, 8, 0x9, 0x0));

  // Read alarm seconds 
  alarmss = ReadDigit((uint8_t*)LCDStringAlarm, 10, 0x5, 0x0);
  alarmss = (uint8_t)(alarmss * 10 + ReadDigit((uint8_t*)LCDStringAlarm, 11, 0x9, 0x0));

  // Set the alarm 
  RTC_AlarmStr.RTC_AlarmTime.RTC_Hours   = alarmhh;
  RTC_AlarmStr.RTC_AlarmTime.RTC_Minutes = alarmmm;
  RTC_AlarmStr.RTC_AlarmTime.RTC_Seconds = alarmss;
  RTC_SetAlarm(RTC_Format_BIN, &RTC_AlarmStr);

  RTC_ITConfig(RTC_IT_ALRA, ENABLE);
  RTC_AlarmCmd(ENABLE);
}
/**
  * @brief  Configures an alarm event to occurs within the current day
  * @param  None
  * @retval None
  */
void Alarm_Adjust(void)
{
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_Print("  Alarm Adjust  ");
  
  // Get the current Time
  RTC_GetTime(RTC_Format_BCD, &RTC_TimeStr);

  RTC_AlarmStr.RTC_AlarmTime.RTC_Hours   = RTC_TimeStr.RTC_Hours;
  RTC_AlarmStr.RTC_AlarmTime.RTC_Minutes = RTC_TimeStr.RTC_Minutes;
  RTC_AlarmStr.RTC_AlarmTime.RTC_Seconds = RTC_TimeStr.RTC_Seconds;
  RTC_SetAlarm(RTC_Format_BCD, &RTC_AlarmStr);

  // Display the current Alarm 
  Alarm_Display();

  // Change the current Alarm 
  Alarm_Regulate();

  // Execute MenuUpdate function to display upper menu 
  MenuUpdate(JOY_UP);
}
/**
  * @brief   Displays alarm time.
  * @param  None
  * @retval None
  */
void Alarm_Display(void)
{
  // Get the current Time
  RTC_GetAlarm(RTC_Format_BCD, &RTC_AlarmStr);

  // Fill the LCDString fields with the current Time
  LCDStringAlarm[4] = (uint8_t)(((uint8_t)(RTC_AlarmStr.RTC_AlarmTime.RTC_Hours & 0xF0) >> 4) + ASCII_NUM_0);
  LCDStringAlarm[5] = (uint8_t)(((uint8_t)(RTC_AlarmStr.RTC_AlarmTime.RTC_Hours & 0x0F)) + ASCII_NUM_0);

  LCDStringAlarm[7] = (uint8_t)(((uint8_t)(RTC_AlarmStr.RTC_AlarmTime.RTC_Minutes & 0xF0) >> 4) + ASCII_NUM_0);
  LCDStringAlarm[8] = (uint8_t)(((uint8_t)(RTC_AlarmStr.RTC_AlarmTime.RTC_Minutes & 0x0F)) + (uint8_t)ASCII_NUM_0);

  LCDStringAlarm[10] = (uint8_t)(((uint8_t)(RTC_AlarmStr.RTC_AlarmTime.RTC_Seconds & 0xF0) >> 4) + ASCII_NUM_0);
  LCDStringAlarm[11] = (uint8_t)(((uint8_t)(RTC_AlarmStr.RTC_AlarmTime.RTC_Seconds & 0x0F)) + ASCII_NUM_0);

  // Print the Time Calendar on the LCD
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_Print((uint8_t*)LCDStringAlarm);

}
/**
  * @brief  Shows alarm time (HH/MM/SS) on LCD.
  * @param  None
  * @retval None
  */
void Alarm_Show(void)
{
  JOYState_TypeDef Key;
  
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_Print("   Alarm Show   ");

  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_Print("               ");
  
  // Wait a press on "UP" pushbuttons 
  while (1)
  {
    // Wait Joystick pressed 
    Key = ReadJoystick();
    if (Key == JOY_UP)
    {
      MenuUpdate(Key);
      return;
    }
    // Display current time 
    Alarm_Display();
  }
}
/**
  * @brief  Return the number of the day in the week.
  * @param  None
  * @retval The number of the day
  */
uint16_t DayOfWeek(uint16_t nYear, uint16_t nMonth, uint16_t nDay)
{
  uint16_t nDayOfWeek = 0;
  uint16_t pnDaysBeforeMonth[14] = {0, 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365};

  // The day of Jan 1, nYear 
  nDayOfWeek = 6 + nYear % 7 + CountOfFeb29(nYear) % 7 + 14; /* + 14 : makes nDayOfWeek >= 0 */

  // The day of nMonth 1, nYear 
  nDayOfWeek += pnDaysBeforeMonth[ nMonth ];
  if ( nMonth > 2 && IsLeapYear(nYear))
  {
    nDayOfWeek++;
  }

  // The day of nMonth nDay, nYear 
  nDayOfWeek += nDay - 1;
  nDayOfWeek %= 7;
  // return the number of the day in the week 
  return nDayOfWeek;
}

/**
  * @brief  Return february days number.
  * @param  None
  * @retval February days number.
  */
uint16_t CountOfFeb29(uint16_t nYear)
{
  uint16_t nCount = 0;

  if (nYear > 0)
  {
    nCount = 1; /* Year 0 is a leap year */
    nYear--;    /* Year nYear is not in the period */
  }
  nCount += nYear / 4 - nYear / 100 + nYear / 400;
  return nCount;
}
/**
  * @brief  Check whether the passed year is Leap or not.
  * @param  None
  * @retval TRUE: leap year
  *         FALSE: not leap year
  */
bool IsLeapYear(u16 nYear)
{
  if (nYear % 4 != 0) return FALSE;
  if (nYear % 100 != 0) return TRUE;
  return (bool)(nYear % 400 == 0);
}

/**
  * @brief  Set the LCD cursor to the specified location
  * @param  ptr : Pointer to string to display on the LCD Glass.
  * @param  Line : line where the cursor will be set (LCD_LINE1 or LCD_LINE2)
  * @param  Offset : is the position offset (only even position are supported)
  * @retval None
  */
void SetCursorPosition(uint8_t* ptr, uint8_t Line, uint8_t Offset)
{
  if ((Offset % 2) == 0)
  {
    LCD_SendByte(COMMAND_TYPE, (uint8_t)(Line + Offset / 2));
  }
  else /* (ColBegin % 2)!= 0*/
  {
    LCD_SendByte(COMMAND_TYPE, (uint8_t)(Line + Offset / 2));
    sprintf((char*)TempVar,"%c", ptr[Offset-1]);
    LCD_Print((uint8_t*)TempVar);
  }
}
/**
  * @brief  Set the LCD cursor to the specified location
  * @param  ptr : Pointer to string to display on the LCD Glass.
  * @param  Line : line where the cursor will be set (LCD_LINE1 or LCD_LINE2)
  * @param  Offset : is the position offset (only even position are supported)
  * @retval None
  */
void ToggleCursorPosition(uint8_t* ptr, uint8_t Line, uint8_t Offset)
{
  SetCursorPosition(ptr, Line, Offset);
  LCD_Print(" ");
  WaitDelay(20);
  
  SetCursorPosition(ptr, Line, Offset);
  sprintf((char*)TempVar,"%c", ptr[Offset]);
  LCD_Print((uint8_t*)TempVar);
  WaitDelay(20);
}



