/**
  ******************************************************************************
  * @file    demo_iddmeasure.c
  * @author  MCD Application Team
  * @version V1.2.1
  * @date    19-May-2011
  * @brief   This file contains all the functions for the IDD measurement demo.
  ******************************************************************************
  * @attention
  *
  */ 

// Includes ------------------------------------------------------------------
#include "menu.h"

// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------
#define EVAL_RESISTOR_R92       217   /* 2.17 ohm  */
#define EVAL_RESISTOR_RATIO     100   /* R91 and R92 are multiplied by 100 */
#define EVAL_MAX9938_GAIN       50    /* Ampli-op gain = 50 */
#define ADC_CONVERT_RATIO       806   /* (3300mV / 0xFFF)* 1000 */
#define ASCII_NUM_0             48

// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------

uint16_t ADCdata = 0;
uint32_t VDDmV = 0;
uint32_t IDDrunuA = 0;
uint8_t LCDString[16] = "  000.000 mA    ";

// Private function prototypes -----------------------------------------------
// Private functions ---------------------------------------------------------

  // * @addtogroup Demo_Functions

/**
  * @brief  IDD measurement program.
  * @param  None
  * @retval None
  */
void IDDMEASURE_Exec(void)
{
  __IO uint8_t current1000, current100, current10, current1, current01, current001 = 0;
  JOYState_TypeDef Key;

  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_Print("   Run Mode    ");
  
  ADC_DeInit(ADC1);
  // Initialize and configure ADC1 
  ADC_Init(ADC1, ADC_ConversionMode_Continuous, ADC_Resolution_12Bit, ADC_Prescaler_2);
  ADC_SamplingTimeConfig(ADC1, ADC_Group_SlowChannels, ADC_SamplingTime_384Cycles);

  // Enable ADC1 
  ADC_Cmd(ADC1, ENABLE);
  
  // Enable ADC1 Channel 19 
  ADC_ChannelCmd(ADC1, ADC_Channel_19, ENABLE);

  // Start ADC1 Conversion using Software trigger
  ADC_SoftwareStartConv(ADC1);

  // Infinite loop
  while (1)
  {
    // Wait Joystick pressed 
    Key = ReadJoystick();
    
    if (Key == JOY_UP)
    {
      ADC_ChannelCmd(ADC1, ADC_Channel_19, DISABLE);
      MenuUpdate(Key);
      return;
    }

    if (Key == JOY_SEL)
    {
      // Clear LCD 
       LCD_Clear();
  
      LCD_SetCursorPos(LCD_LINE1, 0);
      LCD_Print(" IDD Run Mode   ");
      
      while (1)
      {
        // Wait Joystick up pressed to exit 
        Key = ReadJoystick();
        if (Key == JOY_UP)
        {
          MenuUpdate(Key);
          return;
        }
        while (ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == RESET)
        {
        }

        ADCdata = ADC_GetConversionValue(ADC1);

        // Calculate voltage value
        VDDmV = (uint32_t)((uint32_t)ADCdata * (uint32_t)ADC_CONVERT_RATIO) / 1000;

        // calculate the current consumption 
        IDDrunuA = (uint32_t) ((uint32_t)(((VDDmV * 1000 * EVAL_RESISTOR_RATIO) / EVAL_MAX9938_GAIN))\
                                            / (uint32_t)(EVAL_RESISTOR_R92));

        // x  current value
        current1000 = (uint8_t)(IDDrunuA / 100000);
        // Hundreds current value 
        current100 = (uint8_t)((IDDrunuA % 100000) / 10000);
        // Tens current value 
        current10 = (uint8_t)((IDDrunuA % 10000) / 1000);
        // 1mA current value 
        current1 = (uint8_t)((IDDrunuA % 1000) / 100);
        // 0.1 mA current value 
        current01 = (uint8_t)((IDDrunuA % 100 ) / 10);
        // 0.01 mA current value 
        current001 =  (uint8_t)(IDDrunuA % 10);


        // Fill the LCDString fields with the current Voltage 
        if (current1000 == 0)
          LCDString[2] = ' ';
        else
          LCDString[2] = (uint8_t)((uint8_t)(current1000) + ASCII_NUM_0);

        if (current100 == 0 && current1000 == 0)
          LCDString[3] = ' ';
        else
          LCDString[3] = (uint8_t)((uint8_t)(current100) + ASCII_NUM_0);

        LCDString[4] = (uint8_t)((uint8_t)(current10) + ASCII_NUM_0);


        if (LCDString[5] == ',')
        {
          LCDString[5] = '.';
        }
        else
        {
          LCDString[5] = ',';
        }

        LCDString[6] = (uint8_t)((uint8_t)(current1) + ASCII_NUM_0);
        LCDString[7] = (uint8_t)((uint8_t)(current01) + ASCII_NUM_0);
        LCDString[8] = (uint8_t)((uint8_t)(current001) + ASCII_NUM_0);
        LCDString[10] = 'm'; /* mA*/

        disableInterrupts();
        
        // Print the RV2 Voltage on the LCD
        LCD_SetCursorPos(LCD_LINE2, 0);
        LCD_Print(LCDString);
        enableInterrupts();
      }
    }
  }
}


