/**
  ******************************************************************************
  * @file    demo_lowpower.c
  * @author  MCD Application Team
  * @version V1.2.1
  * @date    19-May-2011
  * @brief   This file contains all the functions for the low power demo.
  ******************************************************************************
  * @attention
  *
  */ 

// Includes ------------------------------------------------------------------
#include "menu.h"

// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------
extern RTC_TimeTypeDef   RTC_TimeStr;
extern RTC_AlarmTypeDef  RTC_AlarmStr;

// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------
// Private function prototypes -----------------------------------------------
// Private functions ---------------------------------------------------------
  // * @addtogroup Demo_Functions

/**
  * @brief  Sets the system clock to 16 MHz, using the external 16 MHz oscillator.
  * @param  None
  * @retval None
  */
void SetSysClockTo16(void)
{  
  JOYState_TypeDef Key;
  
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_Print("    RUN Mode   ");
  
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_Print("      16MHz    ");


  // High speed external clock prescaler: 1
  CLK_SYSCLKDivConfig(CLK_SYSCLKDiv_1);

  // Wait a press on "UP" pushbuttons 
  while (1)
  {
    // Wait Joystick pressed 
    Key = ReadJoystick();
    if (Key == JOY_UP)
    {
      MenuUpdate(Key);
      return;
    }
  }
}

/**
  * @brief  Sets the system clock to 8 MHz, using the external 16 MHz oscillator.
  * @param  None
  * @retval None
  */
void SetSysClockTo8(void)
{
  JOYState_TypeDef Key;

  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_Print("    Run Mode   ");
  
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_Print("      8MHz     ");


  // High speed external clock prescaler: 2
  CLK_SYSCLKDivConfig(CLK_SYSCLKDiv_2);

  // Wait a press on "UP" pushbuttons 
  while (1)
  {
    // Wait Joystick pressed 
    Key = ReadJoystick();
    if (Key == JOY_UP)
    {
      MenuUpdate(Key);
      
      // High speed external clock prescaler: 1
      CLK_SYSCLKDivConfig(CLK_SYSCLKDiv_1);
      
      MenuUpdate(Key);
      
      return;
    }
  }
}

/**
  * @brief  Sets the system clock to 4 MHz, using the external 16 MHz oscillator.
  * @param  None
  * @retval None
  */
void SetSysClockTo4(void)
{
  JOYState_TypeDef Key;
  
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_Print("    Run Mode   ");
  
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_Print("      4MHz     ");

  
  // High speed external clock prescaler: 4
  CLK_SYSCLKDivConfig(CLK_SYSCLKDiv_4);
  
  // Wait a press on "UP" pushbuttons 
  while (1)
  {
    // Wait Joystick pressed 
    Key = ReadJoystick();
    
    if (Key == JOY_UP)
    {
      MenuUpdate(Key);
      
      // High speed external clock prescaler: 1
      CLK_SYSCLKDivConfig(CLK_SYSCLKDiv_1);
      
      MenuUpdate(Key);
      
      return;
    }
  }
}
/**
  * @brief  Sets the system clock to 2 MHz, using the external 16 MHz oscillator.
  * @param  None
  * @retval None
  */
void SetSysClockTo2(void)
{
  JOYState_TypeDef Key;
  
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_Print("    Run Mode   ");
  
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_Print("      2MHz     ");

  // High speed external clock prescaler: 8
  CLK_SYSCLKDivConfig(CLK_SYSCLKDiv_8);
  
  // Wait a press on "UP" pushbuttons 
  while (1)
  {
    // Wait Joystick pressed 
    Key = ReadJoystick();
    if (Key == JOY_UP)
    {
      MenuUpdate(Key);
      
      // High speed external clock prescaler: 1
      CLK_SYSCLKDivConfig(CLK_SYSCLKDiv_1);
      
      MenuUpdate(Key);
      
      return;
    }
  }
}

/**
  * @brief  Enters MCU in wait for interrupt(WFI) mode. The wake-up from WFI mode
  *         is performed by an external interrupt.
  * @param  None
  * @retval None
  */
void EnterWFIMode_EXTI(void)
{
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_Print("  Press SEL  ");

  // Initialize I/O in Input Mode with Interrupt 
  STM_EVAL_PBInit(BUTTON_SEL, BUTTON_MODE_EXTI);

  // Disable the TIM2 update Interrupt 
  TIM2_ITConfig(TIM2_IT_Update, DISABLE);

  // Enter Wait for interrupt mode
  wfi();

  // Initialize I/O in Input Mode without Interrupt 
  STM_EVAL_PBInit(BUTTON_SEL, BUTTON_MODE_GPIO);

  // Enable the TIM2 update Interrupt 
  TIM2_ITConfig(TIM2_IT_Update, ENABLE);

  MenuUpdate(JOY_UP);
}

/**
  * @brief  Enters MCU in wait for interrupt(WFI) mode. The wake-up from WFI mode
  *         is performed by an RTC Alarm interrupt.
  * @param  None
  * @retval None
  */
void EnterWFIMode_RTCAlarm(void)
{
  // Get the current Time
  RTC_GetTime(RTC_Format_BCD, &RTC_TimeStr);

  RTC_AlarmStr.RTC_AlarmTime.RTC_Hours   = RTC_TimeStr.RTC_Hours;
  RTC_AlarmStr.RTC_AlarmTime.RTC_Minutes = RTC_TimeStr.RTC_Minutes;
  RTC_AlarmStr.RTC_AlarmTime.RTC_Seconds = RTC_TimeStr.RTC_Seconds;
  RTC_SetAlarm(RTC_Format_BCD, &RTC_AlarmStr);

  // Display the current Alarm 
  Alarm_Display();

  // Change the current Alarm 
  Alarm_Regulate();

  
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_Print("    WFI Mode   ");
  
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_Print("Wait for ALARM ");

  // Disable the TIM2 update Interrupt 
  TIM2_ITConfig(TIM2_IT_Update, DISABLE);

  // Enter Wait for interrupt mode
  wfi();

  // Enable the TIM2 update Interrupt 
  TIM2_ITConfig(TIM2_IT_Update, ENABLE);

  // Execute MenuUpdate function to display upper menu 
  MenuUpdate(JOY_UP);
}

/**
  * @brief  Enters MCU in wait for event (WFE) mode. The wake-up from WFE mode
  *         is performed by an external event.
  * @param  None
  * @retval None
  */
void EnterWFEMode_EXTEvent(void)
{
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_Print("  Press SEL  ");

  // Initialize I/O in Input Mode with Interrupt 
  STM_EVAL_PBInit(BUTTON_SEL, BUTTON_MODE_EXTI);

  // Set the input pin to generate the wake up signal
  WFE_WakeUpSourceEventCmd(WFE_Source_EXTI_EV2, ENABLE);

  // Disable the TIM2 update Interrupt 
  TIM2_ITConfig(TIM2_IT_Update, DISABLE);

  // Enter wait for event mode
  wfe();

  // Enable the TIM2 update Interrupt 
  TIM2_ITConfig(TIM2_IT_Update, ENABLE);

  // Initialize I/O in Input Mode without Interrupt 
  STM_EVAL_PBInit(BUTTON_SEL, BUTTON_MODE_GPIO);

  // Set the input pin to generate the wake up signal
  WFE_WakeUpSourceEventCmd(WFE_Source_EXTI_EV2, DISABLE);

  /* The EXTI_IT_Pin2 interrupts is not served, user has to properly clear
    the corresponding status flags. */
  EXTI_ClearITPendingBit(EXTI_IT_Pin2);

  MenuUpdate(JOY_UP);
}

/**
  * @brief  Enters MCU in wait for event(WFE) mode. The wake-up from WFE mode
  *         is performed by an RTC Alarm event.
  * @param  None
  * @retval None
  */
void EnterWFEMode_COMPEvent(void)
{
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_Print("   Rotate RV2  ");

  // Connect internal reference voltage to COMP1 inverting input 
  COMP_VrefintToCOMP1Connect(ENABLE);

  // close the analog switch number 14 
  SYSCFG_RIAnalogSwitchConfig(RI_AnalogSwitch_14, ENABLE);

  // close the analog switch number 1 
  SYSCFG_RIAnalogSwitchConfig(RI_AnalogSwitch_1, ENABLE);

  // close the I/O switch number 4 
  SYSCFG_RIIOSwitchConfig(RI_IOSwitch_4, ENABLE);

  // Configure the event detection 
  COMP_EdgeConfig(COMP_Selection_COMP1, COMP_Edge_Rising_Falling);

  // Set the input pin to generate the wake up signal
  WFE_WakeUpSourceEventCmd(WFE_Source_ADC1_COMP_EV, ENABLE);

  WaitDelay(50);

  // Clear the comparator1 event flag 
  COMP_ClearFlag(COMP_Selection_COMP1);

  // Enable COMP1 Interrupt 
  COMP_ITConfig(COMP_Selection_COMP1, ENABLE);

  // Disable the TIM2 update Interrupt 
  TIM2_ITConfig(TIM2_IT_Update, DISABLE);

  // Enter Wait for interrupt mode
  wfe();

  // Disable COMP1 Interrupt 
  COMP_ITConfig(COMP_Selection_COMP1, DISABLE);

  // Disable the TIM2 update Interrupt 
  TIM2_ITConfig(TIM2_IT_Update, ENABLE);

  /* The COMP_IT interrupt is not served, user has to properly clear
    the corresponding status flags. */
  COMP_ClearITPendingBit(COMP_Selection_COMP1);

  // Set the input pin to generate the wake up signal
  WFE_WakeUpSourceEventCmd(WFE_Source_ADC1_COMP_EV, DISABLE);

  // Execute MenuUpdate function to display upper menu 
  MenuUpdate(JOY_UP);
}

/**
  * @brief  Enters MCU in Active Halt mode. The wake-up from Active Halt mode is
  *         performed by an external interrupt.
  * @param  None
  * @retval None
  */

void EnterActiveHaltMode_EXTI(void)
{
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_Print("  Press SEL  ");

  // Initialize I/O in Input Mode with Interrupt 
  STM_EVAL_PBInit(BUTTON_SEL, BUTTON_MODE_EXTI);

  // Disable the TIM2 update Interrupt 
  TIM2_ITConfig(TIM2_IT_Update, DISABLE);

  // Enter Halt mode
  halt();

  // Initialize I/O in Input Mode without Interrupt 
  STM_EVAL_PBInit(BUTTON_SEL, BUTTON_MODE_GPIO);

  // Enable the TIM2 update Interrupt 
  TIM2_ITConfig(TIM2_IT_Update, ENABLE);

  MenuUpdate(JOY_UP);
}

/**
  * @brief  Enters MCU in Active Halt mode. The wake-up from Active Halt mode is
  *         performed by an RTC Alarm interrupt.
  * @param  None
  * @retval None
  */
void EnterActiveHaltMode_RTCAlarm(void)
{
  // Get the current Time
  RTC_GetTime(RTC_Format_BCD, &RTC_TimeStr);

  RTC_AlarmStr.RTC_AlarmTime.RTC_Hours   = RTC_TimeStr.RTC_Hours;
  RTC_AlarmStr.RTC_AlarmTime.RTC_Minutes = RTC_TimeStr.RTC_Minutes;
  RTC_AlarmStr.RTC_AlarmTime.RTC_Seconds = RTC_TimeStr.RTC_Seconds;
  RTC_SetAlarm(RTC_Format_BCD, &RTC_AlarmStr);

  // Display the current Alarm 
  Alarm_Display();

  // Change the current Alarm 
  Alarm_Regulate();
  
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_Print("ActiveHalt Mode");
  
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_Print("Wait for ALARM ");

  // Disable the TIM2 update Interrupt 
  TIM2_ITConfig(TIM2_IT_Update, DISABLE);

  // Enter Wait for interrupt mode
  halt();

  // Enable the TIM2 update Interrupt 
  TIM2_ITConfig(TIM2_IT_Update, ENABLE);

  // Execute MenuUpdate function to display upper menu 
  MenuUpdate(JOY_UP);
}

/**
  * @brief  Enters MCU in Halt mode. The wake-up from Halt mode is performed by
  *         an external interrupt.
  * @param  None
  * @retval None
  */
void EnterHaltMode_EXTI(void)
{
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_Print("  Press SEL  ");

  // Initialize I/O in Input Mode with Interrupt 
  STM_EVAL_PBInit(BUTTON_SEL, BUTTON_MODE_EXTI);

  // Disable the TIM2 update Interrupt 
  TIM2_ITConfig(TIM2_IT_Update, DISABLE);

  /* The RTC peripheral is configured to work with LSI clock, so the RTC
     clock should be disabled before execute the halt instruction */
  CLK_PeripheralClockConfig(CLK_Peripheral_RTC,  DISABLE);

  halt();

  // Enable the RTC Clock
  CLK_PeripheralClockConfig(CLK_Peripheral_RTC,  ENABLE);

  // Initialize I/O in Input Mode without Interrupt 
  STM_EVAL_PBInit(BUTTON_SEL, BUTTON_MODE_GPIO);

  // Enable the TIM2 update Interrupt 
  TIM2_ITConfig(TIM2_IT_Update, ENABLE);

  MenuUpdate(JOY_UP);
}

/**
  * @brief  Enters MCU in Halt mode. The wake-up from Halt mode is performed by
  *         an external interrupt.
  * @param  None
  * @retval None
  */
void EnterHaltMode_COMP(void)
{
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_Print("   Rotate RV2  ");

  // Connect internal reference voltage to COMP1 inverting input 
  COMP_VrefintToCOMP1Connect(ENABLE);

  // close the analog switch number 14 
  SYSCFG_RIAnalogSwitchConfig(RI_AnalogSwitch_14, ENABLE);

  // close the analog switch number 1 
  SYSCFG_RIAnalogSwitchConfig(RI_AnalogSwitch_1, ENABLE);

  // close the I/O switch number 4 
  SYSCFG_RIIOSwitchConfig(RI_IOSwitch_4, ENABLE);

  // Configure the event detection 
  COMP_EdgeConfig(COMP_Selection_COMP1, COMP_Edge_Rising_Falling);

  WaitDelay(50);

  // Enable COMP1 Interrupt 
  COMP_ITConfig(COMP_Selection_COMP1, ENABLE);

  // Disable the TIM2 update Interrupt 
  TIM2_ITConfig(TIM2_IT_Update, DISABLE);

  // Enter Wait for interrupt mode
  halt();

  // Disable the TIM2 update Interrupt 
  TIM2_ITConfig(TIM2_IT_Update, ENABLE);

  /* The COMP_IT interrupt is not served, user has to properly clear
    the corresponding status flags. */
  COMP_ClearITPendingBit(COMP_Selection_COMP1);

  // Set the input pin to generate the wake up signal
  WFE_WakeUpSourceEventCmd(WFE_Source_ADC1_COMP_EV, DISABLE);

  // Execute MenuUpdate function to display upper menu 
  MenuUpdate(JOY_UP);
}



