/**
  ******************************************************************************
  * @file    demo_thermometer.c
  * @author  MCD Application Team
  * @version V1.2.1
  * @date    19-May-2011
  * @brief   This file contains all the functions for the Thermometer demo.
  ******************************************************************************
  * @attention
  *
  */ 

// Includes ------------------------------------------------------------------
#include "menu.h"
#include <stdlib.h>

// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------
#define TEMPERATURE_THYS     31
#define TEMPERATURE_TOS      32
// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------
__IO uint8_t TempCelsiusDisplay[]     = "    +abc.d C    ";
__IO uint8_t TempFahrenheitDisplay[]  = "    +abc.d F    ";
__IO uint32_t TempDecimal = 0, TempValueFahrenheit = 0;
__IO uint16_t TempValueCelsius = 0, TempValue = 0;

#ifdef _COSMIC_
extern __IO uint8_t SMbusAlertOccurred @0x310;
#else /* _RAISONANCE_ */
extern __IO uint8_t SMbusAlertOccurred;
#endif /* _COSMIC_ */



// Private function prototypes -----------------------------------------------
// Private functions ---------------------------------------------------------

  // * @addtogroup Demo_Functions

/**
  * @brief  Measure the Temperature.
  * @param  None
  * @retval None
  */
void THERMOMETER_Exec(void)
{
  JOYState_TypeDef Key;

  
  LM75_DeInit();
    
  // Initialize the Temperature Sensor 
  LM75_Init();

  if (LM75_GetStatus() == SUCCESS)
  {
    // Clear LCD 
    LCD_Clear();
    
    I2C_ClearITPendingBit(I2C1, I2C_IT_SMBALERT);
    
    /* Configure the Temperature sensor device STLM75:
      - Thermostat mode Interrupt
      - Fault tolerance: 00
    */
    LM75_WriteConfReg(0x02);
    
    // Configure the THYS and TOS inorder to use the SMbus alert interrupt 
    LM75_WriteReg(LM75_REG_THYS, (uint16_t)(TEMPERATURE_THYS << 8));  /*31C*/
    LM75_WriteReg(LM75_REG_TOS, (uint16_t)(TEMPERATURE_TOS << 8));   /*32C*/
    
    I2C_ClearITPendingBit(I2C1, I2C_IT_SMBALERT);
    SMbusAlertOccurred = 0;
    
    LCD_SetCursorPos(LCD_LINE1, 0);
    LCD_Print("  Temperature  ");

    while (1)
    {
      // Wait Joystick up pressed to exit 
      Key = ReadJoystick();
      if (Key == JOY_UP)
      {
        MenuUpdate(Key);
        return;
      }
      // Get double of Temperature value 
      TempValue = LM75_ReadTemp();

      if (TempValue <= 256)
      {
        // Positive temperature measured 
        TempCelsiusDisplay[4] = '+';
        // Initialize the temperature sensor value
        TempValueCelsius = TempValue;
      }
      else
      {
        // Negative temperature measured 
        TempCelsiusDisplay[4] = '-';
        // Remove temperature value sign 
        TempValueCelsius = 0x200 - TempValue;
      }

      // Calculate temperature digits in C 
      if ((TempValueCelsius & 0x01) == 0x01)
      {
        TempCelsiusDisplay[9] = 0x05 + 0x30;
        TempFahrenheitDisplay[9] = 0x05 + 0x30;
      }
      else
      {
        TempCelsiusDisplay[9] = 0x00 + 0x30;
        TempFahrenheitDisplay[9] = 0x00 + 0x30;
      }
      TempValueCelsius >>= 1;
      TempCelsiusDisplay[5] = (uint8_t)((TempValueCelsius/ 100) + 0x30);
      TempCelsiusDisplay[6] = (uint8_t)(((TempValueCelsius % 100) / 10) + 0x30);
      TempCelsiusDisplay[7] = (uint8_t)(((TempValueCelsius % 100) % 10) + 0x30);

      LCD_SetCursorPos(LCD_LINE2, 0);
      LCD_Print((uint8_t*)TempCelsiusDisplay);

      if (Key == JOY_RIGHT)
      {
      if (TempValue > 256)
      {
        if (((9 * TempValueCelsius) / 5) <= 32)
        {
          // Convert temperature C to Fahrenheit 
          TempValueFahrenheit = abs (32 - ((9 * TempValueCelsius) / 5));

          // Calculate temperature digits in F 
          TempFahrenheitDisplay[5] = (uint8_t)((TempValueFahrenheit / 100) + 0x30);
          TempFahrenheitDisplay[6] = (uint8_t)(((TempValueFahrenheit % 100) / 10) + 0x30);
          TempFahrenheitDisplay[7] = (uint8_t)(((TempValueFahrenheit % 100) % 10) + 0x30);
          // Positive temperature measured 
          TempFahrenheitDisplay[4] = '+';
        }
        else
        {
          // Convert temperature C to Fahrenheit 
          TempValueFahrenheit = abs(((9 * TempValueCelsius) / 5) - 32);
          // Calculate temperature digits in F 
          TempFahrenheitDisplay[5] = (uint8_t)((TempValueFahrenheit / 100) + 0x30);
          TempFahrenheitDisplay[6] = (uint8_t)(((TempValueFahrenheit % 100) / 10) + 0x30);
          TempFahrenheitDisplay[7] = (uint8_t)(((TempValueFahrenheit % 100) % 10) + 0x30);

          // Negative temperature measured 
          TempFahrenheitDisplay[4] = '-';
        }
      }
      else
      {
        // Convert temperature C to Fahrenheit 
        TempValueFahrenheit = ((9 * TempValueCelsius) / 5) + 32;

        // Calculate temperature digits in F 
        TempFahrenheitDisplay[5] = (uint8_t)((TempValueFahrenheit / 100) + 0x30);
        TempFahrenheitDisplay[6] = (uint8_t)(((TempValueFahrenheit % 100) / 10) + 0x30);
        TempFahrenheitDisplay[7] = (uint8_t)(((TempValueFahrenheit % 100) % 10) + 0x30);

        // Positive temperature measured 
        TempFahrenheitDisplay[4] = '+';
      }
      
      LCD_SetCursorPos(LCD_LINE2, 0);
      LCD_Print((uint8_t*)TempFahrenheitDisplay);
      
      WaitDelay(100);
      while (STM_EVAL_PBGetState(BUTTON_RIGHT) == RESET);
      }
      if (SMbusAlertOccurred == 1)
      {
        LCD_SetCursorPos(LCD_LINE1, 0);
        LCD_Print("     Warning   ");
        LCD_SetCursorPos(LCD_LINE2, 0);
        LCD_Print("Temp exceed 32C");
        
        // Get double of Temperature value 
        TempValue = LM75_ReadTemp();      
        while(SMbusAlertOccurred == 1);
        LCD_SetCursorPos(LCD_LINE1, 0);
        LCD_Print("  Temperature  ");
        LCD_SetCursorPos(LCD_LINE2, 0);
        LCD_Print("               ");
        SMbusAlertOccurred = 0;
      }
    }
  }
  else
  {
    LCD_SetCursorPos(LCD_LINE1, 0);
    LCD_Print("ERR: Set JP7 in");
    
    LCD_SetCursorPos(LCD_LINE2, 0);
    LCD_Print("I2C pos. & try ");

    // Wait Joystick pressed 
    Key = ReadJoystick();
    if (Key != JOY_NONE)
    {
      MenuUpdate(Key);
      return;
    }
  }

}



