/**
  ******************************************************************************
  * @file    main.c
  * @author  MCD Application Team
  * @version V1.2.1
  * @date    19-May-2011
  * @brief   Main program body
  ******************************************************************************
  * @attention
  *
  */ 

// Includes ------------------------------------------------------------------
#include "main.h"

  // * @addtogroup DEMO_Functions

// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------
// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------
// Private function prototypes -----------------------------------------------
void PrintLogo(void);

// Private functions ---------------------------------------------------------
// Public functions ----------------------------------------------------------

/**
  * @brief  Main program.
  * @param  None
  * @retval None
  */
void main(void)
{
  JOYState_TypeDef Key;
  uint8_t i;

  Init_Application();

  // Display the welcome text on first LCD line 
  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_Print(" STM8L15x DEMO");
  
  for (i = 1; i < 8; i++)
  {
    PrintLogo();
    WaitDelay(70);
  }
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_Print("               ");
  
  LCD_DisplayLogo(0x93);
  WaitDelay(150);

  // Display the menu 
  MenuDisplay();

  // Executive loop, runs continually after initialization 
  while (1)
  {
    // Menu update managemnt 
    Key = ReadJoystick();

    if (Key != JOY_NONE)
    {
      MenuUpdate(Key);
    }
  }
}

/**
  * @brief  Print ST logo on the LCD
  * @param  None
  * @retval None
  */
void PrintLogo(void)
{
  static uint8_t logo_pos = 0x90;
  static int8_t logo_pos_index = 1;
 
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_Print("               ");
  
  LCD_DisplayLogo(logo_pos);
  logo_pos += logo_pos_index;
  if (logo_pos > 0x95)
  {
    logo_pos -= 2;
    logo_pos_index = -1;
  }
  if (logo_pos < 0x90)
  {
    logo_pos += 2;
    logo_pos_index = 1;
  }
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
  */
void assert_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  // Infinite loop 
  while (1)
  {}
}
#endif



