/**
  ******************************************************************************
  * @file    menu.c
  * @author  MCD Application Team
  * @version V1.2.1
  * @date    19-May-2011
  * @brief   This file includes the menu navigation driver for the STM8L1526-EVAL
  *          Firmware.
  ******************************************************************************
  * @attention
  *
  */ 
// Includes ------------------------------------------------------------------
#include "menu.h"

// Private typedef -----------------------------------------------------------

// Private define ------------------------------------------------------------
#define MENU_TOP_TEXT                    ("  ** MENU **   ")
#define HELP_TEXT                        ("HELP - push SEL")
#define ABOUT_TEXT                       ("     ABOUT     ")
#define THERMOMETER_DEMO_TEXT            ("  THERMOMETER  ")
#define IDDMEASURE_DEMO_TEXT             ("IDD MEASUREMENT")
#define CALENDAR_DEMO_TEXT               ("   CALENDAR    ")
#define LOWPOWER_DEMO_TEXT               ("   LOW POWER   ")
#define VOICE_DEMO_TEXT                  ("     AUDIO     ")

#define TIME_DEMO_TEXT                   ("     TIME      ")
#define TIMESHOW_DEMO_TEXT               ("     SHOW      ")
#define TIMEADJUST_DEMO_TEXT             ("    ADJUST     ")

#define DATE_DEMO_TEXT                   ("     DATE      ")
#define DATESHOW_DEMO_TEXT               ("     SHOW      ")
#define DATEADJUST_DEMO_TEXT             ("    ADJUST     ")

#define ALARM_DEMO_TEXT                  ("     ALARM     ")
#define ALARMSHOW_DEMO_TEXT              ("     SHOW      ")
#define ALARMADJUST_DEMO_TEXT            ("     ADJUST    ")

#define RUN_DEMO_TEXT                    ("    RUN Mode   ")
#define WFI_DEMO_TEXT                    ("    WFI Mode   ")
#define WFE_DEMO_TEXT                    ("    WFE Mode   ")
#define ACTIVEHALT_DEMO_TEXT             ("ActiveHALT Mode")
#define HALT_DEMO_TEXT                   ("    HALT Mode  ")

#define RUN_16MHZ_DEMO_TEXT              ("      16MHz    ")
#define RUN_8MHZ_DEMO_TEXT               ("      8MHz     ")
#define RUN_4MHZ_DEMO_TEXT               ("      4MHz     ")
#define RUN_2MHZ_DEMO_TEXT               ("      2MHz     ")

#define WFI_EXTI_DEMO_TEXT               ("Exit: EXTI     ")
#define WFI_ALARM_DEMO_TEXT              ("Exit: RTC Alarm")

#define WFE_EXTEVENT_DEMO_TEXT           ("Exit:EXT Event ")
#define WFE_COMPEVENT_DEMO_TEXT          ("Exit:COMP Event")

#define ACTIVEHALT_EXTI_DEMO_TEXT        ("Exit: EXTI     ")
#define ACTIVEHALT_ALARM_DEMO_TEXT       ("Exit: RTC Alarm")

#define HALT_EXTI_DEMO_TEXT              ("Exit: EXTI     ")
#define HALT_COMP_DEMO_TEXT              ("Exit:COMP Int. ")

#define VOICERECORDER_DEMO_TEXT          ("Voice Recorder ")
#define VOICEPLAYER_DEMO_TEXT            ("Voice Player   ")


CONST MENU_NEXTSTATE_T MENU_NEXTSTATE[] =
  {
    // STATE                     INPUT           NEXT STATE 
    // MENU Level 1          /* Key */
    { HELP,                      JOY_LEFT,                      ABOUT },
    { HELP,                      JOY_RIGHT,             CALENDAR_DEMO },
    { HELP,                      JOY_SEL,                 EXE_HELP },

    { CALENDAR_DEMO,             JOY_LEFT,                       HELP },
    { CALENDAR_DEMO,             JOY_RIGHT,          THERMOMETER_DEMO },
    { CALENDAR_DEMO,             JOY_SEL,                TIME_DEMO },

    { THERMOMETER_DEMO,          JOY_LEFT,              CALENDAR_DEMO },
    { THERMOMETER_DEMO,          JOY_RIGHT,           IDDMEASURE_DEMO },
    { THERMOMETER_DEMO,          JOY_SEL,          EXE_THERMOMETER },

    { IDDMEASURE_DEMO,           JOY_LEFT,           THERMOMETER_DEMO },
    { IDDMEASURE_DEMO,           JOY_RIGHT,             LOWPOWER_DEMO },
    { IDDMEASURE_DEMO,           JOY_SEL,           EXE_IDDMEASURE },

    { LOWPOWER_DEMO,             JOY_LEFT,            IDDMEASURE_DEMO },
    { LOWPOWER_DEMO,             JOY_RIGHT,                VOICE_DEMO },
    { LOWPOWER_DEMO,             JOY_SEL,                 RUN_DEMO },

    { VOICE_DEMO,                JOY_LEFT,              LOWPOWER_DEMO },
    { VOICE_DEMO,                JOY_RIGHT,                     ABOUT },
    { VOICE_DEMO,                JOY_SEL,       VOICERECORDER_DEMO },

    { ABOUT,                     JOY_LEFT,                 VOICE_DEMO },
    { ABOUT,                     JOY_RIGHT,                      HELP },
    { ABOUT,                     JOY_SEL,                EXE_ABOUT },

    // Calendar DEMO menu Level 2 
    { TIME_DEMO,                 JOY_UP,                CALENDAR_DEMO },
    { TIME_DEMO,                 JOY_LEFT,                 ALARM_DEMO },
    { TIME_DEMO,                 JOY_RIGHT,                 DATE_DEMO },
    { TIME_DEMO,                 JOY_SEL,            TIMESHOW_DEMO },

    { DATE_DEMO,                 JOY_UP,                CALENDAR_DEMO },
    { DATE_DEMO,                 JOY_LEFT,                  TIME_DEMO },
    { DATE_DEMO,                 JOY_RIGHT,                ALARM_DEMO },
    { DATE_DEMO,                 JOY_SEL,            DATESHOW_DEMO },

    { ALARM_DEMO,                JOY_UP,                CALENDAR_DEMO },
    { ALARM_DEMO,                JOY_LEFT,                  DATE_DEMO },
    { ALARM_DEMO,                JOY_RIGHT,                 TIME_DEMO },
    { ALARM_DEMO,                JOY_SEL,           ALARMSHOW_DEMO },

    // Low power DEMO menu Level 2 
    { RUN_DEMO,                  JOY_UP,                LOWPOWER_DEMO },
    { RUN_DEMO,                  JOY_LEFT,                  HALT_DEMO },
    { RUN_DEMO,                  JOY_RIGHT,                  WFI_DEMO },
    { RUN_DEMO,                  JOY_SEL,           RUN_16MHZ_DEMO },

    { WFI_DEMO,                  JOY_UP,                LOWPOWER_DEMO },
    { WFI_DEMO,                  JOY_LEFT,                   RUN_DEMO },
    { WFI_DEMO,                  JOY_RIGHT,                  WFE_DEMO },
    { WFI_DEMO,                  JOY_SEL,            WFI_EXTI_DEMO },

    { WFE_DEMO,                  JOY_UP,                LOWPOWER_DEMO },
    { WFE_DEMO,                  JOY_LEFT,                   WFI_DEMO },
    { WFE_DEMO,                  JOY_RIGHT,           ACTIVEHALT_DEMO },
    { WFE_DEMO,                  JOY_SEL,        WFE_EXTEVENT_DEMO },

    { ACTIVEHALT_DEMO,           JOY_UP,                LOWPOWER_DEMO },
    { ACTIVEHALT_DEMO,           JOY_LEFT,                   WFE_DEMO },
    { ACTIVEHALT_DEMO,           JOY_RIGHT,                 HALT_DEMO },
    { ACTIVEHALT_DEMO,           JOY_SEL,     ACTIVEHALT_EXTI_DEMO },

    { HALT_DEMO,                 JOY_UP,                 LOWPOWER_DEMO},
    { HALT_DEMO,                 JOY_LEFT,            ACTIVEHALT_DEMO },
    { HALT_DEMO,                 JOY_RIGHT,                  RUN_DEMO },
    { HALT_DEMO,                 JOY_SEL,           HALT_EXTI_DEMO },

    // Voice DEMO menu Level 2 
    { VOICERECORDER_DEMO,        JOY_UP,                   VOICE_DEMO },
    { VOICERECORDER_DEMO,        JOY_LEFT,           VOICEPLAYER_DEMO },
    { VOICERECORDER_DEMO,        JOY_RIGHT,          VOICEPLAYER_DEMO },
    { VOICERECORDER_DEMO,        JOY_SEL,        EXE_VOICERECORDER },

    { VOICEPLAYER_DEMO,          JOY_UP,                   VOICE_DEMO },
    { VOICEPLAYER_DEMO,          JOY_LEFT,         VOICERECORDER_DEMO },
    { VOICEPLAYER_DEMO,          JOY_RIGHT,        VOICERECORDER_DEMO },
    { VOICEPLAYER_DEMO,          JOY_SEL,          EXE_VOICEPLAYER },

    // Time DEMO menu Level 3 
    { TIMESHOW_DEMO,             JOY_UP,                    TIME_DEMO },
    { TIMESHOW_DEMO,             JOY_LEFT,            TIMEADJUST_DEMO },
    { TIMESHOW_DEMO,             JOY_RIGHT,           TIMEADJUST_DEMO },
    { TIMESHOW_DEMO,             JOY_SEL,             EXE_TIMESHOW },

    { TIMEADJUST_DEMO,           JOY_UP,                    TIME_DEMO },
    { TIMEADJUST_DEMO,           JOY_LEFT,              TIMESHOW_DEMO },
    { TIMEADJUST_DEMO,           JOY_RIGHT,             TIMESHOW_DEMO },
    { TIMEADJUST_DEMO,           JOY_SEL,           EXE_TIMEADJUST },

    // Date DEMO menu Level 3 
    { DATESHOW_DEMO,             JOY_UP,                    DATE_DEMO },
    { DATESHOW_DEMO,             JOY_LEFT,            DATEADJUST_DEMO },
    { DATESHOW_DEMO,             JOY_RIGHT,           DATEADJUST_DEMO },
    { DATESHOW_DEMO,             JOY_SEL,             EXE_DATESHOW },

    { DATEADJUST_DEMO,           JOY_UP,                    DATE_DEMO },
    { DATEADJUST_DEMO,           JOY_LEFT,              DATESHOW_DEMO },
    { DATEADJUST_DEMO,           JOY_RIGHT,             DATESHOW_DEMO },
    { DATEADJUST_DEMO,           JOY_SEL,           EXE_DATEADJUST },

    // Alarm DEMO menu Level 3 
    { ALARMSHOW_DEMO,            JOY_UP,                   ALARM_DEMO },
    { ALARMSHOW_DEMO,            JOY_LEFT,           ALARMADJUST_DEMO },
    { ALARMSHOW_DEMO,            JOY_RIGHT,          ALARMADJUST_DEMO },
    { ALARMSHOW_DEMO,            JOY_SEL,            EXE_ALARMSHOW },

    { ALARMADJUST_DEMO,          JOY_UP,                   ALARM_DEMO },
    { ALARMADJUST_DEMO,          JOY_LEFT,             ALARMSHOW_DEMO },
    { ALARMADJUST_DEMO,          JOY_RIGHT,            ALARMSHOW_DEMO },
    { ALARMADJUST_DEMO,          JOY_SEL,          EXE_ALARMADJUST },

    // Run DEMO menu Level 3 
    { RUN_16MHZ_DEMO,            JOY_UP,                     RUN_DEMO },
    { RUN_16MHZ_DEMO,            JOY_LEFT,              RUN_2MHZ_DEMO },
    { RUN_16MHZ_DEMO,            JOY_RIGHT,             RUN_8MHZ_DEMO },
    { RUN_16MHZ_DEMO,            JOY_SEL,            EXE_RUN_16MHZ },

    { RUN_8MHZ_DEMO,             JOY_UP,                     RUN_DEMO },
    { RUN_8MHZ_DEMO,             JOY_LEFT,             RUN_16MHZ_DEMO },
    { RUN_8MHZ_DEMO,             JOY_RIGHT,             RUN_4MHZ_DEMO },
    { RUN_8MHZ_DEMO,             JOY_SEL,             EXE_RUN_8MHZ },

    { RUN_4MHZ_DEMO,             JOY_UP,                     RUN_DEMO },
    { RUN_4MHZ_DEMO,             JOY_LEFT,              RUN_8MHZ_DEMO },
    { RUN_4MHZ_DEMO,             JOY_RIGHT,             RUN_2MHZ_DEMO },
    { RUN_4MHZ_DEMO,             JOY_SEL,             EXE_RUN_4MHZ },

    { RUN_2MHZ_DEMO,             JOY_UP,                     RUN_DEMO },
    { RUN_2MHZ_DEMO,             JOY_LEFT,              RUN_4MHZ_DEMO },
    { RUN_2MHZ_DEMO,             JOY_RIGHT,            RUN_16MHZ_DEMO },
    { RUN_2MHZ_DEMO,             JOY_SEL,             EXE_RUN_2MHZ },

    // WFI DEMO menu Level 3 
    { WFI_EXTI_DEMO,             JOY_UP,                     WFI_DEMO },
    { WFI_EXTI_DEMO,             JOY_LEFT,             WFI_ALARM_DEMO },
    { WFI_EXTI_DEMO,             JOY_RIGHT,            WFI_ALARM_DEMO },
    { WFI_EXTI_DEMO,             JOY_SEL,             EXE_WFI_EXTI },

    { WFI_ALARM_DEMO,            JOY_UP,                     WFI_DEMO },
    { WFI_ALARM_DEMO,            JOY_LEFT,              WFI_EXTI_DEMO },
    { WFI_ALARM_DEMO,            JOY_RIGHT,             WFI_EXTI_DEMO },
    { WFI_ALARM_DEMO,            JOY_SEL,            EXE_WFI_ALARM },

    // WFE DEMO menu Level 3 
    { WFE_EXTEVENT_DEMO,         JOY_UP,                     WFE_DEMO },
    { WFE_EXTEVENT_DEMO,         JOY_LEFT,         WFE_COMPEVENT_DEMO },
    { WFE_EXTEVENT_DEMO,         JOY_RIGHT,        WFE_COMPEVENT_DEMO },
    { WFE_EXTEVENT_DEMO,         JOY_SEL,         EXE_WFE_EXTEVENT },

    { WFE_COMPEVENT_DEMO,        JOY_UP,                     WFE_DEMO },
    { WFE_COMPEVENT_DEMO,        JOY_LEFT,          WFE_EXTEVENT_DEMO },
    { WFE_COMPEVENT_DEMO,        JOY_RIGHT,         WFE_EXTEVENT_DEMO },
    { WFE_COMPEVENT_DEMO,        JOY_SEL,        EXE_WFE_COMPEVENT },

    // Active Halt DEMO menu Level 3 
    { ACTIVEHALT_EXTI_DEMO,      JOY_UP,              ACTIVEHALT_DEMO },
    { ACTIVEHALT_EXTI_DEMO,      JOY_LEFT,      ACTIVEHALT_ALARM_DEMO },
    { ACTIVEHALT_EXTI_DEMO,      JOY_RIGHT,     ACTIVEHALT_ALARM_DEMO },
    { ACTIVEHALT_EXTI_DEMO,      JOY_SEL,      EXE_ACTIVEHALT_EXTI },

    { ACTIVEHALT_ALARM_DEMO,     JOY_UP,              ACTIVEHALT_DEMO },
    { ACTIVEHALT_ALARM_DEMO,     JOY_LEFT,       ACTIVEHALT_EXTI_DEMO },
    { ACTIVEHALT_ALARM_DEMO,     JOY_RIGHT,      ACTIVEHALT_EXTI_DEMO },
    { ACTIVEHALT_ALARM_DEMO,     JOY_SEL,     EXE_ACTIVEHALT_ALARM },

    // Halt DEMO menu Level 3 
    { HALT_EXTI_DEMO,            JOY_UP,                    HALT_DEMO },
    { HALT_EXTI_DEMO,            JOY_LEFT,             HALT_COMP_DEMO },
    { HALT_EXTI_DEMO,            JOY_RIGHT,            HALT_COMP_DEMO },
    { HALT_EXTI_DEMO,            JOY_SEL,            EXE_HALT_EXTI },

    { HALT_COMP_DEMO,            JOY_UP,                    HALT_DEMO },
    { HALT_COMP_DEMO,            JOY_LEFT,             HALT_EXTI_DEMO },
    { HALT_COMP_DEMO,            JOY_RIGHT,            HALT_EXTI_DEMO },
    { HALT_COMP_DEMO,            JOY_SEL,            EXE_HALT_COMP },
    // EXE menu 
    { EXE_ABOUT,                 JOY_UP,                        ABOUT },
    { EXE_HELP,                  JOY_UP,                         HELP },
    { EXE_THERMOMETER,           JOY_UP,             THERMOMETER_DEMO },
    { EXE_IDDMEASURE,            JOY_UP,              IDDMEASURE_DEMO },
    { EXE_TIMESHOW,              JOY_UP,                TIMESHOW_DEMO },
    { EXE_TIMEADJUST,            JOY_UP,              TIMEADJUST_DEMO },
    { EXE_DATESHOW,              JOY_UP,                DATESHOW_DEMO },
    { EXE_DATEADJUST,            JOY_UP,              DATEADJUST_DEMO },
    { EXE_ALARMSHOW,             JOY_UP,               ALARMSHOW_DEMO },
    { EXE_ALARMADJUST,           JOY_UP,             ALARMADJUST_DEMO },
    { EXE_RUN_16MHZ,             JOY_UP,               RUN_16MHZ_DEMO },
    { EXE_RUN_8MHZ,              JOY_UP,                RUN_8MHZ_DEMO },
    { EXE_RUN_4MHZ,              JOY_UP,                RUN_4MHZ_DEMO },
    { EXE_RUN_2MHZ,              JOY_UP,                RUN_2MHZ_DEMO },
    { EXE_WFI_EXTI,              JOY_UP,                WFI_EXTI_DEMO },
    { EXE_WFI_ALARM,             JOY_UP,               WFI_ALARM_DEMO },
    { EXE_WFE_EXTEVENT,          JOY_UP,            WFE_EXTEVENT_DEMO },
    { EXE_WFE_COMPEVENT,         JOY_UP,           WFE_COMPEVENT_DEMO },
    { EXE_ACTIVEHALT_EXTI,       JOY_UP,         ACTIVEHALT_EXTI_DEMO },
    { EXE_ACTIVEHALT_ALARM,      JOY_UP,         ACTIVEHALT_EXTI_DEMO },
    { EXE_HALT_EXTI,             JOY_UP,               HALT_EXTI_DEMO },
    { EXE_HALT_COMP,             JOY_UP,               HALT_COMP_DEMO },
    { EXE_VOICERECORDER,         JOY_UP,           VOICERECORDER_DEMO },
    { EXE_VOICEPLAYER,           JOY_UP,             VOICEPLAYER_DEMO },

    // End of table 
    { (STATE_ID_T)0,(JOYState_TypeDef)0,               (STATE_ID_T)0, }
  };

CONST MENU_STATE_T MENU_STATE[] =
  {
    // STATE                     STATE TEXT                FUNCTION 
    // MENU Level 1 
    { HELP,                      HELP_TEXT,                       0 },
    { ABOUT,                     ABOUT_TEXT,                      0 },
    { THERMOMETER_DEMO,          THERMOMETER_DEMO_TEXT,           0 },
    { IDDMEASURE_DEMO,           IDDMEASURE_DEMO_TEXT,            0 },
    { CALENDAR_DEMO,             CALENDAR_DEMO_TEXT,              0 },
    { LOWPOWER_DEMO,             LOWPOWER_DEMO_TEXT,              0 },
    { VOICE_DEMO,                VOICE_DEMO_TEXT,                 0 },
    // Calendar DEMO menu level 2 
    { TIME_DEMO,                 TIME_DEMO_TEXT,                  0 },
    { DATE_DEMO,                 DATE_DEMO_TEXT,                  0 },
    { ALARM_DEMO,                ALARM_DEMO_TEXT,                 0 },

    // Low Power DEMO menu level 2 
    { RUN_DEMO,                 RUN_DEMO_TEXT,                    0 },
    { WFI_DEMO,                 WFI_DEMO_TEXT,                    0 },
    { WFE_DEMO,                 WFE_DEMO_TEXT,                    0 },
    { ACTIVEHALT_DEMO,          ACTIVEHALT_DEMO_TEXT,             0 },
    { HALT_DEMO,                HALT_DEMO_TEXT,                   0 },

    // Voice DEMO menu level 2 
    { VOICERECORDER_DEMO,       VOICERECORDER_DEMO_TEXT,          0 },
    { VOICEPLAYER_DEMO,         VOICEPLAYER_DEMO_TEXT,            0 },

    // Calendar DEMO menu level 3 
    { TIMESHOW_DEMO,             TIMESHOW_DEMO_TEXT,              0 },
    { TIMEADJUST_DEMO,           TIMEADJUST_DEMO_TEXT,            0 },

    { DATESHOW_DEMO,             DATESHOW_DEMO_TEXT,              0 },
    { DATEADJUST_DEMO,           DATEADJUST_DEMO_TEXT,            0 },

    { ALARMSHOW_DEMO,            ALARMSHOW_DEMO_TEXT,             0 },
    { ALARMADJUST_DEMO,          ALARMADJUST_DEMO_TEXT,           0 },

    // Low Power DEMO menu level 3 
    { RUN_16MHZ_DEMO,           RUN_16MHZ_DEMO_TEXT,              0 },
    { RUN_8MHZ_DEMO,            RUN_8MHZ_DEMO_TEXT,               0 },
    { RUN_4MHZ_DEMO,            RUN_4MHZ_DEMO_TEXT,               0 },
    { RUN_2MHZ_DEMO,            RUN_2MHZ_DEMO_TEXT,               0 },

    { WFI_EXTI_DEMO,            WFI_EXTI_DEMO_TEXT,               0 },
    { WFI_ALARM_DEMO,           WFI_ALARM_DEMO_TEXT,              0 },

    { WFE_EXTEVENT_DEMO,        WFE_EXTEVENT_DEMO_TEXT,           0 },
    { WFE_COMPEVENT_DEMO,       WFE_COMPEVENT_DEMO_TEXT,          0 },

    { ACTIVEHALT_EXTI_DEMO,     ACTIVEHALT_EXTI_DEMO_TEXT,        0 },
    { ACTIVEHALT_ALARM_DEMO,    ACTIVEHALT_ALARM_DEMO_TEXT,       0 },

    { HALT_EXTI_DEMO,           HALT_EXTI_DEMO_TEXT,              0 },
    { HALT_COMP_DEMO,           HALT_COMP_DEMO_TEXT,              0 },

    // Low Power DEMO menu level 3 
    { RUN_16MHZ_DEMO,           RUN_16MHZ_DEMO_TEXT,              0 },
    { RUN_8MHZ_DEMO,            RUN_8MHZ_DEMO_TEXT,               0 },
    { RUN_4MHZ_DEMO,            RUN_4MHZ_DEMO_TEXT,               0 },
    { RUN_2MHZ_DEMO,            RUN_2MHZ_DEMO_TEXT,               0 },
    { WFI_EXTI_DEMO,            WFI_EXTI_DEMO_TEXT,               0 },
    { WFI_ALARM_DEMO,           WFI_ALARM_DEMO_TEXT,              0 },
    { WFE_EXTEVENT_DEMO,        WFE_EXTEVENT_DEMO_TEXT,           0 },
    { WFE_COMPEVENT_DEMO,       WFE_COMPEVENT_DEMO_TEXT,          0 },
    { ACTIVEHALT_EXTI_DEMO,     ACTIVEHALT_EXTI_DEMO_TEXT,        0 },
    { ACTIVEHALT_ALARM_DEMO,    ACTIVEHALT_ALARM_DEMO_TEXT,       0 },
    { HALT_EXTI_DEMO,           HALT_EXTI_DEMO_TEXT,              0 },
    { HALT_COMP_DEMO,           HALT_COMP_DEMO_TEXT,              0 },

    // EXE menu 
    { EXE_HELP,                  0,                       HELP_Exec },
    { EXE_ABOUT,                 0,                      ABOUT_Exec },
    { EXE_THERMOMETER,           0,                THERMOMETER_Exec },
    { EXE_IDDMEASURE,            0,                 IDDMEASURE_Exec },
    { EXE_TIMESHOW,              0,                       Time_Show },
    { EXE_TIMEADJUST,            0,                     Time_Adjust },
    { EXE_DATESHOW,              0,                       Date_Show },
    { EXE_DATEADJUST,            0,                     Date_Adjust },
    { EXE_ALARMSHOW,             0,                      Alarm_Show },
    { EXE_ALARMADJUST,           0,                    Alarm_Adjust },
    { EXE_RUN_16MHZ,             0,                 SetSysClockTo16 },
    { EXE_RUN_8MHZ,              0,                  SetSysClockTo8 },
    { EXE_RUN_4MHZ,              0,                  SetSysClockTo4 },
    { EXE_RUN_2MHZ,              0,                  SetSysClockTo2 },
    { EXE_WFI_EXTI,              0,               EnterWFIMode_EXTI },
    { EXE_WFI_ALARM,             0,           EnterWFIMode_RTCAlarm },
    { EXE_WFE_EXTEVENT,          0,           EnterWFEMode_EXTEvent },
    { EXE_WFE_COMPEVENT,         0,          EnterWFEMode_COMPEvent },
    { EXE_ACTIVEHALT_EXTI,       0,        EnterActiveHaltMode_EXTI },
    { EXE_ACTIVEHALT_ALARM,      0,    EnterActiveHaltMode_RTCAlarm },
    { EXE_HALT_EXTI,             0,              EnterHaltMode_EXTI },
    { EXE_HALT_COMP,             0,              EnterHaltMode_COMP },
    { EXE_VOICERECORDER,         0,              VOICERECORDER_Exec },
    { EXE_VOICEPLAYER,           0,                VOICEPLAYER_Exec },

    // End of table 
    { (STATE_ID_T)0,            0,              0 }
  };

// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------
STATE_ID_T PreviousTopState, State;
void (*pStateFunc)(void);

// Private function prototypes -----------------------------------------------
// Private functions ---------------------------------------------------------

/**
  * @brief  Initializes the navigation menu.
  * @param  None
  * @retval None
  */   
void MenuInit(void)
{
  PreviousTopState = (STATE_ID_T)0;
  State = HELP;
  pStateFunc = 0;
}

/**
  * @brief  Updates the menu states and display the new menu on the LCD.
  * @param  ujoystick: new joystick state
  * @retval None
  */    
void MenuUpdate(JOYState_TypeDef ujoystick)
{
  uint8_t i;
  STATE_ID_T NextState;

  NextState = (STATE_ID_T)0;

  // Search if the next menu state exists 
  for (i = 0; MENU_NEXTSTATE[i].State; i++)
  {
    if (MENU_NEXTSTATE[i].State == State && MENU_NEXTSTATE[i].Key == ujoystick)
    {
      NextState = MENU_NEXTSTATE[i].NextState;
      break;
    }
  }

  // if next state exists 
  if (NextState)
  {
    // Update the previous top state 
    switch (ujoystick)
    {
      case JOY_UP:
        PreviousTopState = (STATE_ID_T)0;
        for (i = 0; MENU_NEXTSTATE[i].State; i++)
        {
          if ((MENU_NEXTSTATE[i].State == NextState) && (MENU_NEXTSTATE[i].Key == JOY_UP))
          {
            PreviousTopState = MENU_NEXTSTATE[i].NextState;
            break;
          }
        }
        break;

      case JOY_SEL:
        // If a sub-menu is entered, the PreviousTopState is the current state 
        PreviousTopState = State;
        break;

      default:
        break;
    }
    State = NextState; /* Update current state */

    MenuDisplay(); /* Display the new menu on LCD */

    // Execute the function of state if there is one
    for (i = 0; MENU_STATE[i].State; i++)
    {
      if (MENU_STATE[i].State == State)
      {
        pStateFunc = MENU_STATE[i].pFunc;
        if (pStateFunc) /* Check if there is a state function to be executed */
        {
          pStateFunc(); /* Execute the state function */
          pStateFunc = 0;
        }
      }
    }
  }
}

/**
  * @brief  Displays the current menu level on the LCD
  * @param  None
  * @retval None
  */   
void MenuDisplay(void)
{

  uint8_t i;
  
  // Clear LCD 
   LCD_Clear();
  
  WaitDelay(20);
  // Display menu line 1 
  if (PreviousTopState)
  {
    for (i = 0; MENU_STATE[i].State; i++)
    {
      if (MENU_STATE[i].State == PreviousTopState)
      {
        if (MENU_STATE[i].pText)
        {
          LCD_SetCursorPos(LCD_LINE1, 0);
          LCD_Print((uint8_t*)MENU_STATE[i].pText);
        }
        break;
      }
    }
  }
  else
  {
    // Menu top level reached 
    LCD_SetCursorPos(LCD_LINE1, 0);
    LCD_Print(MENU_TOP_TEXT);
  }

  // Display menu line 2 
  for (i = 0; MENU_STATE[i].State; i++)
  {
    if (MENU_STATE[i].State == State)
    {
      if (MENU_STATE[i].pText)
      {
        LCD_SetCursorPos(LCD_LINE2, 0);
        LCD_Print((uint8_t*)MENU_STATE[i].pText);
      }
      break;
    }
  }
}

