/**
  ******************************************************************************
  * @file    stm8l15x_it.c
  * @author  MCD Application Team
  * @version V1.2.1
  * @date    19-May-2011
  * @brief   Main Interrupt Service Routines.
  *          This file provides template for all peripherals interrupt service routine.
  ******************************************************************************
  * @attention
  *
  */ 

// Includes ------------------------------------------------------------------
#include "stm8l15x_it.h"
#include "alarm.h"
#include "menu.h"
// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------
// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------
__IO uint8_t GlobalCounter1 = 0;
__IO uint16_t GlobalCounter2 = 0;
__IO uint8_t count = 0;

__IO uint32_t SPIFlashWriteAddr = RECORD_READ_WRITED_ADDR;/* SPI Flash write address */
__IO uint8_t WriteOnTheFlash = 0;
__IO uint16_t FlashCounter = 0;

uint32_t RecordDataLength = 0;     /* Voice recorder global data length */
#ifdef _COSMIC_
__IO uint8_t VoiceRecTab0[PAGE_SIZE] @0x100;
__IO uint8_t VoiceRecTab1[PAGE_SIZE] @0x200;
__IO uint8_t SMbusAlertOccurred @0x310;
#elif defined (_RAISONANCE_)
at 0x100 __IO uint8_t VoiceRecTab0[PAGE_SIZE];
at 0x200 __IO uint8_t VoiceRecTab1[PAGE_SIZE];
at 0x310 __IO uint8_t SMbusAlertOccurred;
#elif defined (_IAR_)
__IO __no_init uint8_t VoiceRecTab0[PAGE_SIZE]  @0x100;
__IO __no_init uint8_t VoiceRecTab1[PAGE_SIZE] @0x200;
__IO __no_init uint8_t SMbusAlertOccurred @0x310;
#endif /* _COSMIC_ */

uint8_t* Tab;                           /* Pointer on VoiceRec_Tab(0-1) */
uint8_t TabIndex = 0;                   /* Variable to choice between VoiceRec_Tab(0-1) */
__IO uint8_t SampleN_1Value = 0x80;     /* ADC (N-1) sample converted data value */
__IO uint8_t DACData = 0;
__IO uint16_t RecordSample = 0;         /* Recorded data index in VoiceRec_Tab(0-1) */

extern __IO uint8_t VoiceRecStop;
extern __IO uint8_t VoicePlayStop;
extern __IO uint8_t  AlarmPlayStop;
extern __IO uint8_t RecordStatus;             /* Voice recorder global Status */
extern __IO uint8_t PlayStatus;               /* Voice recorder global Status */
extern __IO uint8_t AlarmPlayStatus;          /* Alarm Player  global Status */
extern __IO uint32_t WaveDataLength;          /* Wave player global data length */
extern uint32_t SelectedMaxRecordDataLength; /* Selected max SPI Falsh size dedicated */
extern __IO uint32_t WaveCounter;  /* Wave player and voice recorder global index */
extern CONST uint8_t AlarmWave[];
// Private function prototypes -----------------------------------------------
void Delay(__IO uint32_t nCount);
// Private functions ---------------------------------------------------------
/**
  * @brief  Inserts a delay time.
  * @param  nCount: specifies the delay time length.
  * @retval None
  */
void Delay(__IO uint32_t nCount)
{
  __IO uint32_t index = 0;
  for (index = (0x60 * nCount); index != 0; index--)
  {}
}
// Public functions ----------------------------------------------------------

 // @addtogroup IT_Functions
#ifdef _COSMIC_
/**
  * @brief Dummy interrupt routine
  * @par Parameters:
  * None
  * @retval 
  * None
*/
INTERRUPT_HANDLER(NonHandledInterrupt,0)
{
    /* In order to detect unexpected events during development,
       it is recommended to set a breakpoint on the following instruction.
    */
}
#endif

/**
  * @brief  TRAP interrupt routine
  * @param  None
  * @retval None
*/
INTERRUPT_HANDLER_TRAP(TRAP_IRQHandler)
{
  /* In order to detect unexpected events during development,
     it is recommended to set a breakpoint on the following instruction.
  */
}

/**
  * @brief FLASH Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(FLASH_IRQHandler,1)
{
  /* In order to detect unexpected events during development,
     it is recommended to set a breakpoint on the following instruction.
  */
}

/**
  * @brief DMA1 channel0 and channel1 Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(DMA1_CHANNEL0_1_IRQHandler,2)
{
  /* In order to detect unexpected events during development,
     it is recommended to set a breakpoint on the following instruction.
  */
}

/**
  * @brief DMA1 channel2 and channel3 Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(DMA1_CHANNEL2_3_IRQHandler,3)
{
  /* In order to detect unexpected events during development,
     it is recommended to set a breakpoint on the following instruction.
  */
}

/**
  * @brief RTC / CSS_LSE Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(RTC_CSSLSE_IRQHandler,4)
{
  // Select button to stop Playing 
  STM_EVAL_PBInit(BUTTON_UP, BUTTON_MODE_EXTI);
  TIM2_ITConfig(TIM2_IT_Update, DISABLE);
  STM_EVAL_LEDOn(LED1);
  STM_EVAL_LEDOn(LED2);
  STM_EVAL_LEDOn(LED3);
  STM_EVAL_LEDOn(LED4);
  for (count = 0; count < 30; count++)
  {
    STM_EVAL_LEDToggle(LED1);
    STM_EVAL_LEDToggle(LED2);
    STM_EVAL_LEDToggle(LED3);
    STM_EVAL_LEDToggle(LED4);
    Delay(300);
  }

  ALARMPLAYER_Exec();
  // Select button to stop Playing 
  STM_EVAL_PBInit(BUTTON_UP, BUTTON_MODE_GPIO);
  TIM2_ITConfig(TIM2_IT_Update, ENABLE);
  RTC_ClearITPendingBit(RTC_IT_ALRA);
}

/**
  * @brief External IT PORTE/F and PVD Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(EXTIE_F_PVD_IRQHandler,5)
{
  /* In order to detect unexpected events during development,
     it is recommended to set a breakpoint on the following instruction.
  */
}

/**
  * @brief External IT PORTB / PORTG Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(EXTIB_G_IRQHandler,6)
{
  /* In order to detect unexpected events during development,
     it is recommended to set a breakpoint on the following instruction.
  */
}

/**
  * @brief External IT PORTD /PORTH Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(EXTID_H_IRQHandler,7)
{
  /* In order to detect unexpected events during development,
     it is recommended to set a breakpoint on the following instruction.
  */
}

/**
  * @brief External IT PIN0 Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(EXTI0_IRQHandler,8)
{
  /* In order to detect unexpected events during development,
     it is recommended to set a breakpoint on the following instruction.
  */
}

/**
  * @brief External IT PIN1 Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(EXTI1_IRQHandler,9)
{
  /* In order to detect unexpected events during development,
     it is recommended to set a breakpoint on the following instruction.
  */
}

/**
  * @brief External IT PIN2 Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(EXTI2_IRQHandler,10)
{
  /* In WFI mode The EXTI_IT_Pin1 interrupts is served, user has to
   properly clear the corresponding status flags. */
  EXTI_ClearITPendingBit(EXTI_IT_Pin2);
}

/**
  * @brief External IT PIN3 Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(EXTI3_IRQHandler,11)
{
  /* In order to detect unexpected events during development,
     it is recommended to set a breakpoint on the following instruction.
  */
}

/**
  * @brief External IT PIN4 Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(EXTI4_IRQHandler,12)
{
  /* In order to detect unexpected events during development,
     it is recommended to set a breakpoint on the following instruction.
  */
}

/**
  * @brief External IT PIN5 Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(EXTI5_IRQHandler,13)
{
  /* In order to detect unexpected events during development,
     it is recommended to set a breakpoint on the following instruction.
  */
}

/**
  * @brief External IT PIN6 Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(EXTI6_IRQHandler,14)
{
  /* In order to detect unexpected events during development,
     it is recommended to set a breakpoint on the following instruction.
  */
}

/**
  * @brief External IT PIN7 Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(EXTI7_IRQHandler,15)
{
  if (RecordStatus)
  {
    VoiceRecStop = SET;
  }
  if (PlayStatus)
  {
    VoicePlayStop = SET;
  }
  if (AlarmPlayStatus)
  {
    AlarmPlayStop = SET;
  }
  EXTI_ClearITPendingBit(EXTI_IT_Pin7);
}

/**
  * @brief LCD /AES Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(LCD_AES_IRQHandler,16)
{
  /* In order to detect unexpected events during development,
     it is recommended to set a breakpoint on the following instruction.
  */
}

/**
  * @brief CLK switch/CSS/TIM1 break Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(SWITCH_CSS_BREAK_DAC_IRQHandler,17)
{
  /* In order to detect unexpected events during development,
     it is recommended to set a breakpoint on the following instruction.
  */
}

/**
  * @brief ADC1/Comparator Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(ADC1_COMP_IRQHandler,18)
{
  /* In order to detect unexpected events during development,
     it is recommended to set a breakpoint on the following instruction.
  */
  // Clear the comparator1 event flag 
  COMP_ClearFlag(COMP_Selection_COMP1);
}

/**
  * @brief TIM2 Update/Overflow/Trigger/Break /USART2 TX Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(TIM2_UPD_OVF_TRG_BRK_USART2_TX_IRQHandler,19)
{
  /* In order to detect unexpected events during development,
     it is recommended to set a breakpoint on the following instruction.
  */
  GlobalCounter1++;
  GlobalCounter2++;
  if (GlobalCounter1 == 4)
  {
    GlobalCounter1 = 0;
    GTimeStamp++;
  }
  if (GlobalCounter2 == 100)
  {
    STM_EVAL_LEDOff(LED1);
    STM_EVAL_LEDOn(LED2);
    STM_EVAL_LEDOn(LED3);
    STM_EVAL_LEDOn(LED4);
    STM_EVAL_LEDToggle(LED5);
  }
  if (GlobalCounter2 == 200)
  {
    STM_EVAL_LEDOn(LED1);
    STM_EVAL_LEDOff(LED2);
    STM_EVAL_LEDOn(LED3);
    STM_EVAL_LEDOn(LED4);
  }
  if (GlobalCounter2 == 300)
  {
    STM_EVAL_LEDOn(LED1);
    STM_EVAL_LEDOn(LED2);
    STM_EVAL_LEDOff(LED3);
    STM_EVAL_LEDOn(LED4);
    STM_EVAL_LEDToggle(LED5);
  }
  if (GlobalCounter2 == 400)
  {
    STM_EVAL_LEDOn(LED1);
    STM_EVAL_LEDOn(LED2);
    STM_EVAL_LEDOn(LED3);
    STM_EVAL_LEDOff(LED4);
    GlobalCounter2 = 0;
  }
  TIM2_ClearFlag(TIM2_FLAG_Update);
}

/**
  * @brief Timer2 Capture/Compare / USART2 RX Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(TIM2_CC_USART2_RX_IRQHandler,20)
{
  /* In order to detect unexpected events during development,
     it is recommended to set a breakpoint on the following instruction.
  */
}

/**
  * @brief Timer3 Update/Overflow/Trigger/Break Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(TIM3_UPD_OVF_TRG_BRK_USART3_TX_IRQHandler,21)
{
  /* In order to detect unexpected events during development,
     it is recommended to set a breakpoint on the following instruction.
  */
}

/**
  * @brief Timer3 Capture/Compare /USART3 RX Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(TIM3_CC_USART3_RX_IRQHandler,22)
{
  /* In order to detect unexpected events during development,
     it is recommended to set a breakpoint on the following instruction.
  */
}

/**
  * @brief TIM1 Update/Overflow/Trigger/Commutation Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(TIM1_UPD_OVF_TRG_COM_IRQHandler,23)
{
  uint8_t SampleNValue = 0;

  if (RecordStatus)
  {
    // Get the ADC conversion value for sample N 
    SampleNValue = (uint8_t)ADC_GetConversionValue(ADC1);
    
    // If we haven't reach the max allowed recording SPI Flash size 
    if (WaveCounter < SelectedMaxRecordDataLength)
    {
      // Select the VoiceRec_Tab destination table 
      if (TabIndex == 0)
      {
        Tab = (uint8_t*)VoiceRecTab0;
      }
      else
      {
        Tab = (uint8_t*)VoiceRecTab1;
      }

      // Store the converted sample N value 
      Tab[RecordSample++] = SampleNValue;

      // Set actuel N sample value as N-1 sample for next record 
      SampleN_1Value = SampleNValue;

      // If the VoiceRec_Tab destination table is full 
      if (RecordSample == (PAGE_SIZE))
      {
        // save in the SPI Flash
        WriteOnTheFlash = ENABLE;

        // generate an event to start saving in the SPI Flash
        TIM1_GenerateEvent(TIM1_EventSource_CC1);

        // Change the VoiceRec_Tab destination table 
        TabIndex ^= 1 ;

        // Reset RecordSample counter 
        RecordSample = 0;
      }
      // Increment WaveCounter which count the recorded wave length 
      WaveCounter++;
    }
  }
  // Clear TIM1 interrupt 
  TIM1_ClearITPendingBit(TIM1_IT_Update);
}

/**
  * @brief TIM1 Capture/Compare Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(TIM1_CC_IRQHandler,24)
{
  if (WriteOnTheFlash == ENABLE)
  {
    // Load SPI Flash with the actual VoiceRec_Tab 
    sFLASH_WriteBuffer(Tab, SPIFlashWriteAddr, PAGE_SIZE);

    // Increment SPIFlashWriteAddr by 256
    SPIFlashWriteAddr += PAGE_SIZE;

    WriteOnTheFlash = DISABLE;

    FlashCounter++;
  }
  // Clear TIM1 interrupt 
  TIM1_ClearITPendingBit(TIM1_IT_CC1);
}

/**
  * @brief TIM4 Update/Overflow/Trigger Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(TIM4_UPD_OVF_TRG_IRQHandler,25)
{
  // Wave playing phase 
  if (PlayStatus)
  {
    // Read a byte from SPI Flash and output it on the DAC 
    DACData = sFLASH_ReadByte();

    DAC_SetChannel1Data(DAC_Align_8b_R, DACData );

    // Increment wave player index 
    WaveCounter++;
  }

  // Alarm  playing phase 
  if (AlarmPlayStatus)
  {
    DAC_SetChannel1Data(DAC_Align_8b_R, AlarmWave[WaveCounter]);

    // Increment wave player index 
    WaveCounter++;
  }

  // Clear TIM4 interrupt 
  TIM4_ClearITPendingBit(TIM4_IT_Update);

}

/**
  * @brief SPI1 Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(SPI1_IRQHandler,26)
{
  /* In order to detect unexpected events during development,
     it is recommended to set a breakpoint on the following instruction.
  */
}

/**
  * @brief USART1 TX / TIM5 Update/Overflow/Trigger/Break Interrupt  routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(USART1_TX_TIM5_UPD_OVF_TRG_BRK_IRQHandler,27)
{
  /* In order to detect unexpected events during development,
     it is recommended to set a breakpoint on the following instruction.
  */
}

/**
  * @brief USART1 RX / Timer5 Capture/Compare Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(USART1_RX_TIM5_CC_IRQHandler,28)
{
  /* In order to detect unexpected events during development,
     it is recommended to set a breakpoint on the following instruction.
  */
}

/**
  * @brief I2C1 / SPI2 Interrupt routine.
  * @param  None
  * @retval None
  */
INTERRUPT_HANDLER(I2C1_SPI2_IRQHandler,29)
{
  /* In order to detect unexpected events during development,
     it is recommended to set a breakpoint on the following instruction.
  */
  // Check on I2C1 SMBALERT flag and clear it 
  if (I2C_GetITStatus(I2C1, I2C_IT_SMBALERT))
  {
    I2C_ClearITPendingBit(I2C1, I2C_IT_SMBALERT);
    SMbusAlertOccurred++;
  }

  // Check on I2C1 Time out flag and clear it 
  if (I2C_GetITStatus(I2C1, I2C_IT_TIMEOUT))
  {
    I2C_ClearITPendingBit(I2C1, I2C_IT_TIMEOUT);
  }
  // Check on I2C1 Arbitration Lost flag and clear it 
  if (I2C_GetITStatus(I2C1, I2C_IT_ARLO))
  {
    I2C_ClearITPendingBit(I2C1, I2C_IT_ARLO);
  } 

  // Check on I2C1 PEC error flag and clear it 
  if (I2C_GetITStatus(I2C1, I2C_IT_PECERR))
  {
    I2C_ClearITPendingBit(I2C1, I2C_IT_PECERR);
  } 
  // Check on I2C1 Overrun/Underrun error flag and clear it 
  if (I2C_GetITStatus(I2C1, I2C_IT_OVR))
  {
    I2C_ClearITPendingBit(I2C1, I2C_IT_OVR);
  } 
  // Check on I2C1 Acknowledge failure error flag and clear it 
  if (I2C_GetITStatus(I2C1, I2C_IT_AF))
  {
    I2C_ClearITPendingBit(I2C1, I2C_IT_AF);
  }
  // Check on I2C1 Bus error flag and clear it 
  if (I2C_GetITStatus(I2C1, I2C_IT_BERR))
  {
    I2C_ClearITPendingBit(I2C1, I2C_IT_BERR);
  }   
}

